@@ -99,18 +99,16 @@ void api_send(const char *msg)
     }
 
     char *name = api_get_nick();
-    char  timefrmt[TIME_STR_SIZE];
 
     if (name == NULL) {
         return;
     }
 
     self_window = get_active_window();
-    get_time_str(timefrmt, sizeof(timefrmt));
 
     strncpy((char *) self_window->chatwin->line, msg, sizeof(self_window->chatwin->line));
     add_line_to_hist(self_window->chatwin);
-    int id = line_info_add(self_window, timefrmt, name, NULL, OUT_MSG, 0, 0, "%s", msg);
+    int id = line_info_add(self_window, true, name, NULL, OUT_MSG, 0, 0, "%s", msg);
     cqueue_add(self_window->chatwin->cqueue, msg, strlen(msg), OUT_MSG, id);
     free(name);
 }
@@ -156,7 +154,7 @@ void cmd_run(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX
             error_str = "Only one argument allowed.";
         }
 
-        line_info_add(self, NULL, NULL, NULL, SYS_MSG, 0, 0, error_str);
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, error_str);
         return;
     }
 
@@ -165,7 +163,7 @@ void cmd_run(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX
     if (fp == NULL) {
         error_str = "Path does not exist.";
 
-        line_info_add(self, NULL, NULL, NULL, SYS_MSG, 0, 0, error_str);
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, error_str);
         return;
     }
 
@@ -175,36 +173,36 @@ void cmd_run(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX
 
 void invoke_autoruns(WINDOW *window, ToxWindow *self)
 {
-    struct dirent *dir;
-    char    abspath_buf[PATH_MAX + 1], err_buf[PATH_MAX + 1];
-    size_t  path_len;
-    DIR    *d;
-    FILE   *fp;
+    char abspath_buf[PATH_MAX + 256];
+    char err_buf[PATH_MAX + 128];
 
     if (user_settings->autorun_path[0] == '\0') {
         return;
     }
 
-    d = opendir(user_settings->autorun_path);
+    DIR *d = opendir(user_settings->autorun_path);
 
     if (d == NULL) {
-        snprintf(err_buf, PATH_MAX + 1, "Autorun path does not exist: %s", user_settings->autorun_path);
+        snprintf(err_buf, sizeof(err_buf), "Autorun path does not exist: %s", user_settings->autorun_path);
         api_display(err_buf);
         return;
     }
 
+    struct dirent *dir = NULL;
+
     cur_window  = window;
+
     self_window = self;
 
     while ((dir = readdir(d)) != NULL) {
-        path_len = strlen(dir->d_name);
+        size_t path_len = strlen(dir->d_name);
 
         if (!strcmp(dir->d_name + path_len - 3, ".py")) {
-            snprintf(abspath_buf, PATH_MAX + 1, "%s%s", user_settings->autorun_path, dir->d_name);
-            fp = fopen(abspath_buf, "r");
+            snprintf(abspath_buf, sizeof(abspath_buf), "%s%s", user_settings->autorun_path, dir->d_name);
+            FILE *fp = fopen(abspath_buf, "r");
 
             if (fp == NULL) {
-                snprintf(err_buf, PATH_MAX + 1, "Invalid path: %s", abspath_buf);
+                snprintf(err_buf, sizeof(err_buf), "Invalid path: %s", abspath_buf);
                 api_display(err_buf);
                 continue;
             }
