@@ -20,26 +20,26 @@
  *
  */
 
-#include <string.h>
-#include <stdint.h>
-#include <stdlib.h>
-#include <time.h>
 #include <arpa/inet.h>
 #include <assert.h>
+#include <stdint.h>
+#include <stdlib.h>
+#include <string.h>
+#include <time.h>
 
 #include <tox/tox.h>
 
-#include "toxic.h"
-#include "windows.h"
+#include "avatars.h"
 #include "chat.h"
 #include "friendlist.h"
-#include "misc_tools.h"
-#include "line_info.h"
-#include "settings.h"
-#include "notify.h"
 #include "help.h"
+#include "line_info.h"
 #include "log.h"
-#include "avatars.h"
+#include "misc_tools.h"
+#include "notify.h"
+#include "settings.h"
+#include "toxic.h"
+#include "windows.h"
 
 #ifdef AUDIO
 #include "audio_call.h"
@@ -116,8 +116,8 @@ static void realloc_blocklist(int n)
 void kill_friendlist(ToxWindow *self)
 {
     for (size_t i = 0; i < Friends.max_idx; ++i) {
-        if (Friends.list[i].active && Friends.list[i].group_invite.key != NULL) {
-            free(Friends.list[i].group_invite.key);
+        if (Friends.list[i].active && Friends.list[i].conference_invite.key != NULL) {
+            free(Friends.list[i].conference_invite.key);
         }
     }
 
@@ -127,6 +127,20 @@ void kill_friendlist(ToxWindow *self)
     del_window(self);
 }
 
+static void clear_blocklist_index(size_t idx)
+{
+    Blocked.list[idx] = (BlockedFriend) {
+        0
+    };
+}
+
+static void clear_friendlist_index(size_t idx)
+{
+    Friends.list[idx] = (ToxicFriend) {
+        0
+    };
+}
+
 /* Saves the blocklist to path. If there are no items in the blocklist the
  * empty file will be removed.
  *
@@ -160,8 +174,7 @@ static int save_blocklist(char *path)
                 continue;
             }
 
-            BlockedFriend tmp;
-            memset(&tmp, 0, sizeof(BlockedFriend));
+            BlockedFriend tmp = {0};
             tmp.namelength = htons(Blocked.list[i].namelength);
             memcpy(tmp.name, Blocked.list[i].name, Blocked.list[i].namelength + 1);  // Include null byte
             memcpy(tmp.pub_key, Blocked.list[i].pub_key, TOX_PUBLIC_KEY_SIZE);
@@ -187,13 +200,21 @@ static int save_blocklist(char *path)
         return 0;
     }
 
-    char temp_path[strlen(path) + strlen(TEMP_BLOCKLIST_EXT) + 1];
-    snprintf(temp_path, sizeof(temp_path), "%s%s", path, TEMP_BLOCKLIST_EXT);
+    size_t temp_buf_size = strlen(path) + strlen(TEMP_BLOCKLIST_EXT) + 1;
+    char *temp_path = malloc(temp_buf_size);
+
+    if (temp_path == NULL) {
+        free(data);
+        return -1;
+    }
+
+    snprintf(temp_path, temp_buf_size, "%s%s", path, TEMP_BLOCKLIST_EXT);
 
     FILE *fp = fopen(temp_path, "wb");
 
     if (fp == NULL) {
         free(data);
+        free(temp_path);
         return -1;
     }
 
@@ -201,6 +222,7 @@ static int save_blocklist(char *path)
         fprintf(stderr, "Failed to write blocklist data.\n");
         fclose(fp);
         free(data);
+        free(temp_path);
         return -1;
     }
 
@@ -208,9 +230,12 @@ static int save_blocklist(char *path)
     free(data);
 
     if (rename(temp_path, path) != 0) {
+        free(temp_path);
         return -1;
     }
 
+    free(temp_path);
+
     return 0;
 }
 
@@ -235,15 +260,22 @@ int load_blocklist(char *path)
         return -1;
     }
 
-    char data[len];
+    char *data = malloc(len);
+
+    if (data == NULL) {
+        fclose(fp);
+        return -1;
+    }
 
     if (fread(data, len, 1, fp) != 1) {
         fclose(fp);
+        free(data);
         return -1;
     }
 
     if (len % sizeof(BlockedFriend) != 0) {
         fclose(fp);
+        free(data);
         return -1;
     }
 
@@ -251,12 +283,9 @@ int load_blocklist(char *path)
     Blocked.max_idx = num;
     realloc_blocklist(num);
 
-    int i;
-
-    for (i = 0; i < num; ++i) {
-        BlockedFriend tmp;
-        memset(&tmp, 0, sizeof(BlockedFriend));
-        memset(&Blocked.list[i], 0, sizeof(BlockedFriend));
+    for (int i = 0; i < num; ++i) {
+        BlockedFriend tmp = {0};
+        clear_blocklist_index(i);
 
         memcpy(&tmp, data + i * sizeof(BlockedFriend), sizeof(BlockedFriend));
         Blocked.list[i].namelength = ntohs(tmp.namelength);
@@ -279,6 +308,8 @@ int load_blocklist(char *path)
     }
 
     fclose(fp);
+    free(data);
+
     sort_blocklist_index();
 
     return 0;
@@ -308,7 +339,9 @@ void sort_friendlist_index(void)
         }
     }
 
-    qsort(Friends.index, Friends.num_friends, sizeof(uint32_t), index_name_cmp);
+    if (Friends.num_friends > 0) {
+        qsort(Friends.index, Friends.num_friends, sizeof(uint32_t), index_name_cmp);
+    }
 }
 
 static int index_name_cmp_block(const void *n1, const void *n2)
@@ -344,6 +377,10 @@ static void update_friend_last_online(uint32_t num, time_t timestamp)
 static void friendlist_onMessage(ToxWindow *self, Tox *m, uint32_t num, Tox_Message_Type type, const char *str,
                                  size_t length)
 {
+    UNUSED_VAR(self);
+    UNUSED_VAR(type);
+    UNUSED_VAR(length);
+
     if (num >= Friends.max_idx) {
         return;
     }
@@ -360,16 +397,15 @@ static void friendlist_onMessage(ToxWindow *self, Tox *m, uint32_t num, Tox_Mess
     char nick[TOX_MAX_NAME_LENGTH];
     get_nick_truncate(m, nick, num);
 
-    char timefrmt[TIME_STR_SIZE];
-    get_time_str(timefrmt, sizeof(timefrmt));
-
-    line_info_add(prompt, timefrmt, nick, NULL, IN_MSG, 0, 0, "%s", str);
-    line_info_add(prompt, NULL, NULL, NULL, SYS_MSG, 0, RED, "* Warning: Too many windows are open.");
+    line_info_add(prompt, true, nick, NULL, IN_MSG, 0, 0, "%s", str);
+    line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, RED, "* Warning: Too many windows are open.");
     sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
 }
 
 static void friendlist_onConnectionChange(ToxWindow *self, Tox *m, uint32_t num, Tox_Connection connection_status)
 {
+    UNUSED_VAR(self);
+
     if (num >= Friends.max_idx) {
         return;
     }
@@ -380,7 +416,7 @@ static void friendlist_onConnectionChange(ToxWindow *self, Tox *m, uint32_t num,
         ++Friends.num_online;
 
         if (avatar_send(m, num) == -1) {
-            fprintf(stderr, "avatar_send failed for friend %d\n", num);
+            fprintf(stderr, "avatar_send failed for friend %u\n", num);
         }
     }
 
@@ -392,6 +428,9 @@ static void friendlist_onConnectionChange(ToxWindow *self, Tox *m, uint32_t num,
 
 static void friendlist_onNickChange(ToxWindow *self, Tox *m, uint32_t num, const char *nick, size_t length)
 {
+    UNUSED_VAR(self);
+    UNUSED_VAR(length);
+
     if (num >= Friends.max_idx) {
         return;
     }
@@ -411,7 +450,9 @@ static void friendlist_onNickChange(ToxWindow *self, Tox *m, uint32_t num, const
     tox_self_get_address(m, (uint8_t *) myid);
 
     if (strcmp(oldname, newnamecpy) != 0) {
-        rename_logfile(oldname, newnamecpy, myid, Friends.list[num].pub_key, Friends.list[num].chatwin);
+        if (rename_logfile(oldname, newnamecpy, myid, Friends.list[num].pub_key, Friends.list[num].chatwin) != 0) {
+            fprintf(stderr, "Failed to rename friend chat log from `%s` to `%s`\n", oldname, newnamecpy);
+        }
     }
 
     sort_friendlist_index();
@@ -419,6 +460,9 @@ static void friendlist_onNickChange(ToxWindow *self, Tox *m, uint32_t num, const
 
 static void friendlist_onStatusChange(ToxWindow *self, Tox *m, uint32_t num, Tox_User_Status status)
 {
+    UNUSED_VAR(self);
+    UNUSED_VAR(m);
+
     if (num >= Friends.max_idx) {
         return;
     }
@@ -428,6 +472,8 @@ static void friendlist_onStatusChange(ToxWindow *self, Tox *m, uint32_t num, Tox
 
 static void friendlist_onStatusMessageChange(ToxWindow *self, uint32_t num, const char *note, size_t length)
 {
+    UNUSED_VAR(self);
+
     if (length > TOX_MAX_STATUS_MESSAGE_LENGTH || num >= Friends.max_idx) {
         return;
     }
@@ -438,8 +484,10 @@ static void friendlist_onStatusMessageChange(ToxWindow *self, uint32_t num, cons
 
 void friendlist_onFriendAdded(ToxWindow *self, Tox *m, uint32_t num, bool sort)
 {
+    UNUSED_VAR(self);
+
     realloc_friends(Friends.max_idx + 1);
-    memset(&Friends.list[Friends.max_idx], 0, sizeof(ToxicFriend));
+    clear_friendlist_index(Friends.max_idx);
 
     uint32_t i;
 
@@ -473,10 +521,11 @@ void friendlist_onFriendAdded(ToxWindow *self, Tox *m, uint32_t num, bool sort)
 
         update_friend_last_online(i, t);
 
-        char tempname[TOX_MAX_NAME_LENGTH] = {0};
-        get_nick_truncate(m, tempname, num);
-        snprintf(Friends.list[i].name, sizeof(Friends.list[i].name), "%s", tempname);
-        Friends.list[i].namelength = strlen(Friends.list[i].name);
+        char tempname[TOX_MAX_NAME_LENGTH + 1];
+        int name_len = get_nick_truncate(m, tempname, num);
+        memcpy(Friends.list[i].name, tempname, name_len);
+        Friends.list[i].name[name_len] = 0;
+        Friends.list[i].namelength = name_len;
 
         if (i == Friends.max_idx) {
             ++Friends.max_idx;
@@ -495,10 +544,10 @@ void friendlist_onFriendAdded(ToxWindow *self, Tox *m, uint32_t num, bool sort)
 }
 
 /* Puts blocked friend back in friendlist. fnum is new friend number, bnum is blocked number. */
-static void friendlist_add_blocked(Tox *m, uint32_t fnum, uint32_t bnum)
+static void friendlist_add_blocked(uint32_t fnum, uint32_t bnum)
 {
     realloc_friends(Friends.max_idx + 1);
-    memset(&Friends.list[Friends.max_idx], 0, sizeof(ToxicFriend));
+    clear_friendlist_index(Friends.max_idx);
 
     int i;
 
@@ -536,6 +585,11 @@ static void friendlist_add_blocked(Tox *m, uint32_t fnum, uint32_t bnum)
 static void friendlist_onFileRecv(ToxWindow *self, Tox *m, uint32_t num, uint32_t filenum,
                                   uint64_t file_size, const char *filename, size_t name_length)
 {
+    UNUSED_VAR(self);
+    UNUSED_VAR(file_size);
+    UNUSED_VAR(filename);
+    UNUSED_VAR(name_length);
+
     if (num >= Friends.max_idx) {
         return;
     }
@@ -554,15 +608,21 @@ static void friendlist_onFileRecv(ToxWindow *self, Tox *m, uint32_t num, uint32_
     char nick[TOX_MAX_NAME_LENGTH];
     get_nick_truncate(m, nick, num);
 
-    line_info_add(prompt, NULL, NULL, NULL, SYS_MSG, 0, RED,
+    line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, RED,
                   "* File transfer from %s failed: too many windows are open.", nick);
 
     sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
 }
 
-static void friendlist_onGroupInvite(ToxWindow *self, Tox *m, int32_t num, uint8_t type, const char *group_pub_key,
-                                     uint16_t length)
+static void friendlist_onConferenceInvite(ToxWindow *self, Tox *m, int32_t num, uint8_t type,
+        const char *conference_pub_key,
+        uint16_t length)
 {
+    UNUSED_VAR(self);
+    UNUSED_VAR(type);
+    UNUSED_VAR(conference_pub_key);
+    UNUSED_VAR(length);
+
     if (num >= Friends.max_idx) {
         return;
     }
@@ -579,14 +639,14 @@ static void friendlist_onGroupInvite(ToxWindow *self, Tox *m, int32_t num, uint8
     char nick[TOX_MAX_NAME_LENGTH];
     get_nick_truncate(m, nick, num);
 
-    line_info_add(prompt, NULL, NULL, NULL, SYS_MSG, 0, RED,
-                  "* Group chat invite from %s failed: too many windows are open.", nick);
+    line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, RED,
+                  "* Conference chat invite from %s failed: too many windows are open.", nick);
 
     sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
 }
 
 /* move friendlist/blocklist cursor up and down */
-static void select_friend(ToxWindow *self, wint_t key, int *selected, int num)
+static void select_friend(wint_t key, int *selected, int num)
 {
     if (num <= 0) {
         return;
@@ -626,11 +686,11 @@ static void delete_friend(Tox *m, uint32_t f_num)
         }
     }
 
-    if (Friends.list[f_num].group_invite.key != NULL) {
-        free(Friends.list[f_num].group_invite.key);
+    if (Friends.list[f_num].conference_invite.key != NULL) {
+        free(Friends.list[f_num].conference_invite.key);
     }
 
-    memset(&Friends.list[f_num], 0, sizeof(ToxicFriend));
+    clear_friendlist_index(f_num);
 
     int i;
 
@@ -656,7 +716,7 @@ static void delete_friend(Tox *m, uint32_t f_num)
 }
 
 /* activates delete friend popup */
-static void del_friend_activate(ToxWindow *self, Tox *m, uint32_t f_num)
+static void del_friend_activate(uint32_t f_num)
 {
     PendingDelete.popup = newwin(3, 22 + TOXIC_MAX_NAME_LENGTH, 8, 8);
     PendingDelete.active = true;
@@ -666,9 +726,9 @@ static void del_friend_activate(ToxWindow *self, Tox *m, uint32_t f_num)
 static void delete_blocked_friend(uint32_t bnum);
 
 /* deactivates delete friend popup and deletes friend if instructed */
-static void del_friend_deactivate(ToxWindow *self, Tox *m, wint_t key)
+static void del_friend_deactivate(Tox *m, wint_t key)
 {
-    if (key == 'y') {
+    if (key == L'y') {
         if (blocklist_view == 0) {
             delete_friend(m, PendingDelete.num);
             sort_friendlist_index();
@@ -679,7 +739,11 @@ static void del_friend_deactivate(ToxWindow *self, Tox *m, wint_t key)
     }
 
     delwin(PendingDelete.popup);
-    memset(&PendingDelete, 0, sizeof(PendingDelete));
+
+    PendingDelete = (struct PendingDel) {
+        0
+    };
+
     clear();
     refresh();
 }
@@ -717,7 +781,7 @@ static void draw_del_popup(void)
 /* deletes contact from blocked list */
 static void delete_blocked_friend(uint32_t bnum)
 {
-    memset(&Blocked.list[bnum], 0, sizeof(BlockedFriend));
+    clear_blocklist_index(bnum);
 
     int i;
 
@@ -740,12 +804,12 @@ static void delete_blocked_friend(uint32_t bnum)
 /* deletes contact from friendlist and puts in blocklist */
 void block_friend(Tox *m, uint32_t fnum)
 {
-    if (Friends.num_friends <= 0) {
+    if (Friends.num_friends == 0) {
         return;
     }
 
     realloc_blocklist(Blocked.max_idx + 1);
-    memset(&Blocked.list[Blocked.max_idx], 0, sizeof(BlockedFriend));
+    clear_blocklist_index(Blocked.max_idx);
 
     int i;
 
@@ -787,35 +851,38 @@ static void unblock_friend(Tox *m, uint32_t bnum)
     uint32_t friendnum = tox_friend_add_norequest(m, (uint8_t *) Blocked.list[bnum].pub_key, &err);
 
     if (err != TOX_ERR_FRIEND_ADD_OK) {
-        line_info_add(prompt, NULL, NULL, NULL, SYS_MSG, 0, 0, "Failed to unblock friend (error %d)", err);
+        line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to unblock friend (error %d)", err);
         return;
     }
 
-    friendlist_add_blocked(m, friendnum, bnum);
+    friendlist_add_blocked(friendnum, bnum);
     delete_blocked_friend(bnum);
     sort_blocklist_index();
     sort_friendlist_index();
 }
 
-static void friendlist_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
+/*
+ * Return true if input is recognized by handler
+ */
+static bool friendlist_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
 {
 
     if (self->help->active) {
         help_onKey(self, key);
-        return;
+        return true;
     }
 
-    if (key == 'h') {
+    if (key == L'h') {
         help_init_menu(self);
-        return;
+        return true;
     }
 
     if (!blocklist_view && !Friends.num_friends && (key != KEY_RIGHT && key != KEY_LEFT)) {
-        return;
+        return true;
     }
 
     if (blocklist_view && !Blocked.num_blocked && (key != KEY_RIGHT && key != KEY_LEFT)) {
-        return;
+        return true;
     }
 
     int f = 0;
@@ -828,19 +895,19 @@ static void friendlist_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
 
     /* lock screen and force decision on deletion popup */
     if (PendingDelete.active) {
-        if (key == 'y' || key == 'n') {
-            del_friend_deactivate(self, m, key);
+        if (key == L'y' || key == L'n') {
+            del_friend_deactivate(m, key);
         }
 
-        return;
+        return true;
     }
 
     if (key == ltr) {
-        return;
+        return true;
     }
 
     switch (key) {
-        case '\r':
+        case L'\r':
             if (blocklist_view) {
                 break;
             }
@@ -853,17 +920,17 @@ static void friendlist_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
                 set_active_window_index(Friends.list[f].chatwin);
             } else {
                 const char *msg = "* Warning: Too many windows are open.";
-                line_info_add(prompt, NULL, NULL, NULL, SYS_MSG, 0, RED, msg);
+                line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, RED, msg);
                 sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
             }
 
             break;
 
         case KEY_DC:
-            del_friend_activate(self, m, f);
+            del_friend_activate(f);
             break;
 
-        case 'b':
+        case L'b':
             if (!blocklist_view) {
                 block_friend(m, f);
             } else {
@@ -879,19 +946,23 @@ static void friendlist_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
 
         default:
             if (blocklist_view == 0) {
-                select_friend(self, key, &Friends.num_selected, Friends.num_friends);
+                select_friend(key, &Friends.num_selected, Friends.num_friends);
             } else {
-                select_friend(self, key, &Blocked.num_selected, Blocked.num_blocked);
+                select_friend(key, &Blocked.num_selected, Blocked.num_blocked);
             }
 
             break;
     }
+
+    return true;
 }
 
 #define FLIST_OFST 6    /* Accounts for space at top and bottom */
 
 static void blocklist_onDraw(ToxWindow *self, Tox *m, int y2, int x2)
 {
+    UNUSED_VAR(m);
+
     wattron(self->window, A_BOLD);
     wprintw(self->window, " Blocked: ");
     wattroff(self->window, A_BOLD);
@@ -948,7 +1019,7 @@ static void blocklist_onDraw(ToxWindow *self, Tox *m, int y2, int x2)
         wmove(self->window, y2 - 1, 1);
 
         wattron(self->window, A_BOLD);
-        wprintw(self->window, "Key: ");
+        wprintw(self->window, "Public key: ");
         wattroff(self->window, A_BOLD);
 
         int i;
@@ -983,6 +1054,8 @@ static void friendlist_onDraw(ToxWindow *self, Tox *m)
     wprintw(self->window, "key for help\n\n");
     wattroff(self->window, COLOR_PAIR(CYAN));
 
+    draw_window_bar(self);
+
     if (blocklist_view == 1) {
         blocklist_onDraw(self, m, y2, x2);
         return;
@@ -995,7 +1068,7 @@ static void friendlist_onDraw(ToxWindow *self, Tox *m)
     wprintw(self->window, " Online: ");
     wattroff(self->window, A_BOLD);
 
-    wprintw(self->window, "%d/%d \n\n", Friends.num_online, Friends.num_friends);
+    wprintw(self->window, "%zu/%zu \n\n", Friends.num_online, Friends.num_friends);
 
     if ((y2 - FLIST_OFST) <= 0) {
         return;
@@ -1024,9 +1097,9 @@ static void friendlist_onDraw(ToxWindow *self, Tox *m)
         int num_selected = Friends.num_selected;
         pthread_mutex_unlock(&Winthread.lock);
 
-        bool f_selected = false;
-
         if (is_active) {
+            bool f_selected = false;
+
             if (i == num_selected) {
                 wattron(self->window, A_BOLD);
                 wprintw(self->window, " > ");
@@ -1173,7 +1246,7 @@ static void friendlist_onDraw(ToxWindow *self, Tox *m)
         wmove(self->window, y2 - 1, 1);
 
         wattron(self->window, A_BOLD);
-        wprintw(self->window, "Key: ");
+        wprintw(self->window, "Public key: ");
         wattroff(self->window, A_BOLD);
 
         int i;
@@ -1191,6 +1264,21 @@ static void friendlist_onDraw(ToxWindow *self, Tox *m)
     }
 }
 
+void friendlist_onInit(ToxWindow *self, Tox *m)
+{
+    UNUSED_VAR(m);
+
+    int x2;
+    int y2;
+    getmaxyx(self->window, y2, x2);
+
+    if (y2 <= 0 || x2 <= 0) {
+        exit_toxic_err("failed in friendlist_onInit", FATALERR_CURSES);
+    }
+
+    self->window_bar = subwin(self->window, WINDOW_BAR_HEIGHT, x2, y2 - 2, 0);
+}
+
 void disable_chatwin(uint32_t f_num)
 {
     Friends.list[f_num].chatwin = -1;
@@ -1199,13 +1287,12 @@ void disable_chatwin(uint32_t f_num)
 #ifdef AUDIO
 static void friendlist_onAV(ToxWindow *self, ToxAV *av, uint32_t friend_number, int state)
 {
-    assert(0);
+    UNUSED_VAR(self);
 
     if (friend_number >= Friends.max_idx) {
         return;
     }
 
-    assert(0);
     Tox *m = toxav_get_tox(av);
 
     if (Friends.list[friend_number].chatwin == -1) {
@@ -1217,10 +1304,10 @@ static void friendlist_onAV(ToxWindow *self, ToxAV *av, uint32_t friend_number,
         } else {
             char nick[TOX_MAX_NAME_LENGTH];
             get_nick_truncate(m, nick, Friends.list[friend_number].num);
-            line_info_add(prompt, NULL, NULL, NULL, SYS_MSG, 0, 0, "Audio action from: %s!", nick);
+            line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, 0, "Audio action from: %s!", nick);
 
             const char *errmsg = "* Warning: Too many windows are open.";
-            line_info_add(prompt, NULL, NULL, NULL, SYS_MSG, 0, RED, errmsg);
+            line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, RED, errmsg);
 
             sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
         }
@@ -1240,6 +1327,26 @@ Tox_Connection get_friend_connection_status(uint32_t friendnumber)
     return Friends.list[friendnumber].connection_status;
 }
 
+/*
+ * Returns true if friend associated with `public_key` is in the block list.
+ *
+ * `public_key` must be at least TOX_PUBLIC_KEY_SIZE bytes.
+ */
+bool friend_is_blocked(const char *public_key)
+{
+    for (size_t i = 0; i < Blocked.max_idx; ++i) {
+        if (!Blocked.list[i].active) {
+            continue;
+        }
+
+        if (memcmp(public_key, Blocked.list[i].pub_key, TOX_PUBLIC_KEY_SIZE) == 0) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 ToxWindow *new_friendlist(void)
 {
     ToxWindow *ret = calloc(1, sizeof(ToxWindow));
@@ -1248,8 +1355,9 @@ ToxWindow *new_friendlist(void)
         exit_toxic_err("failed in new_friendlist", FATALERR_MEMORY);
     }
 
-    ret->is_friendlist = true;
+    ret->type = WINDOW_TYPE_FRIEND_LIST;
 
+    ret->onInit = &friendlist_onInit;
     ret->onKey = &friendlist_onKey;
     ret->onDraw = &friendlist_onDraw;
     ret->onFriendAdded = &friendlist_onFriendAdded;
@@ -1259,7 +1367,7 @@ ToxWindow *new_friendlist(void)
     ret->onStatusChange = &friendlist_onStatusChange;
     ret->onStatusMessageChange = &friendlist_onStatusMessageChange;
     ret->onFileRecv = &friendlist_onFileRecv;
-    ret->onGroupInvite = &friendlist_onGroupInvite;
+    ret->onConferenceInvite = &friendlist_onConferenceInvite;
 
 #ifdef AUDIO
     ret->onInvite = &friendlist_onAV;
@@ -1273,7 +1381,6 @@ ToxWindow *new_friendlist(void)
     ret->onEnd = &friendlist_onAV;
 
     ret->is_call = false;
-    ret->device_selection[0] = ret->device_selection[1] = -1;
 #endif /* AUDIO */
 
     ret->num = -1;
@@ -1286,6 +1393,6 @@ ToxWindow *new_friendlist(void)
     }
 
     ret->help = help;
-    strcpy(ret->name, "contacts");
+    strcpy(ret->name, "Contacts");
     return ret;
 }
