@@ -25,9 +25,9 @@
 
 #include <time.h>
 
+#include "file_transfers.h"
 #include "toxic.h"
 #include "windows.h"
-#include "file_transfers.h"
 
 struct LastOnline {
     uint64_t last_on;
@@ -35,7 +35,7 @@ struct LastOnline {
     char hour_min_str[TIME_STR_SIZE];    /* holds 12/24-hour time string e.g. "10:43 PM" */
 };
 
-struct GroupChatInvite {
+struct ConferenceInvite {
     char *key;
     uint16_t length;
     uint8_t type;
@@ -57,7 +57,7 @@ typedef struct {
     Tox_User_Status status;
 
     struct LastOnline last_online;
-    struct GroupChatInvite group_invite;
+    struct ConferenceInvite conference_invite;
 
     struct FileTransfer file_receiver[MAX_FILES];
     struct FileTransfer file_sender[MAX_FILES];
@@ -82,6 +82,7 @@ typedef struct {
 } FriendsList;
 
 ToxWindow *new_friendlist(void);
+void friendlist_onInit(ToxWindow *self, Tox *m);
 void disable_chatwin(uint32_t f_num);
 int get_friendnum(uint8_t *name);
 int load_blocklist(char *data);
@@ -93,4 +94,11 @@ Tox_Connection get_friend_connection_status(uint32_t friendnumber);
 /* sorts friendlist_index first by connection status then alphabetically */
 void sort_friendlist_index(void);
 
+/*
+ * Returns true if friend associated with `public_key` is in the block list.
+ *
+ * `public_key` must be at least TOX_PUBLIC_KEY_SIZE bytes.
+ */
+bool friend_is_blocked(const char *public_key);
+
 #endif /* end of include guard: FRIENDLIST_H */
