@@ -20,22 +20,22 @@
  *
  */
 
+#include <assert.h>
+#include <stdarg.h>
+#include <stdbool.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdio.h>
-#include <stdbool.h>
-#include <unistd.h>
-#include <stdarg.h>
-#include <time.h>
-#include <assert.h>
 #include <sys/stat.h>
+#include <time.h>
+#include <unistd.h>
 
-#include "notify.h"
 #include "audio_device.h"
-#include "settings.h"
 #include "line_info.h"
 #include "misc_tools.h"
-#include "xtra.h"
+#include "notify.h"
+#include "settings.h"
+#include "x11focus.h"
 
 #if defined(AUDIO) || defined(SOUND_NOTIFY)
 #ifdef __APPLE__
@@ -60,7 +60,7 @@
 
 #define MAX_BOX_MSG_LEN 127
 #define SOUNDS_SIZE 10
-#define ACTIVE_NOTIFS_MAX 50
+#define ACTIVE_NOTIFS_MAX 10
 
 extern struct user_settings *user_settings;
 
@@ -101,6 +101,17 @@ static struct _ActiveNotifications {
 /**********************************************************************************/
 /**********************************************************************************/
 
+static void clear_actives_index(size_t idx)
+{
+    if (actives[idx].id_indicator) {
+        *actives[idx].id_indicator = -1;
+    }
+
+    actives[idx] = (struct _ActiveNotifications) {
+        0
+    };
+}
+
 /* coloured tab notifications: primary notification type */
 static void tab_notify(ToxWindow *self, uint64_t flags)
 {
@@ -115,6 +126,8 @@ static void tab_notify(ToxWindow *self, uint64_t flags)
     } else if ((flags & NT_WNDALERT_2) && (!self->alert || self->alert > WINDOW_ALERT_1)) {
         self->alert = WINDOW_ALERT_2;
     }
+
+    ++self->pending_messages;
 }
 
 static bool notifications_are_disabled(uint64_t flags)
@@ -169,7 +182,7 @@ void m_open_device(void)
     }
 
     /* Blah error check */
-    open_primary_device(output, &Control.device_idx, 48000, 20, 1);
+    open_output_device(&Control.device_idx, 48000, 20, 1);
 
     device_opened = true;
 }
@@ -193,7 +206,7 @@ void graceful_clear(void)
     while (1) {
         int i;
 
-        for (i = 0; i < ACTIVE_NOTIFS_MAX; i ++) {
+        for (i = 0; i < ACTIVE_NOTIFS_MAX; ++i) {
             if (actives[i].active) {
 #ifdef BOX_NOTIFY
 
@@ -213,7 +226,7 @@ void graceful_clear(void)
                     stop_sound(i);
                 } else {
                     if (!is_playing(actives[i].source)) {
-                        memset(&actives[i], 0, sizeof(struct _ActiveNotifications));
+                        clear_actives_index(i);
                     } else {
                         break;
                     }
@@ -227,7 +240,7 @@ void graceful_clear(void)
             return;
         }
 
-        usleep(1000);
+        sleep_thread(1000L);
     }
 
     control_unlock();
@@ -235,7 +248,7 @@ void graceful_clear(void)
 
 void *do_playing(void *_p)
 {
-    (void)_p;
+    UNUSED_VAR(_p);
 
     while (true) {
         control_lock();
@@ -249,7 +262,7 @@ void *do_playing(void *_p)
         bool test_active_notify = false;
         int i;
 
-        for (i = 0; i < ACTIVE_NOTIFS_MAX; i ++) {
+        for (i = 0; i < ACTIVE_NOTIFS_MAX; ++i) {
 
             if (actives[i].looping) {
                 has_looping = true;
@@ -270,7 +283,7 @@ void *do_playing(void *_p)
                     alSourceStop(actives[i].source);
                     alDeleteSources(1, &actives[i].source);
                     alDeleteBuffers(1, &actives[i].buffer);
-                    memset(&actives[i], 0, sizeof(struct _ActiveNotifications));
+                    clear_actives_index(i);
                 }
             }
 
@@ -289,7 +302,7 @@ void *do_playing(void *_p)
                     alSourceStop(actives[i].source);
                     alDeleteSources(1, &actives[i].source);
                     alDeleteBuffers(1, &actives[i].buffer);
-                    memset(&actives[i], 0, sizeof(struct _ActiveNotifications));
+                    clear_actives_index(i);
                 }
             }
 
@@ -305,7 +318,7 @@ void *do_playing(void *_p)
         has_looping = false;
 
         control_unlock();
-        usleep(10000);
+        sleep_thread(10000L);
     }
 
     pthread_exit(NULL);
@@ -315,7 +328,7 @@ int play_source(uint32_t source, uint32_t buffer, bool looping)
 {
     int i = 0;
 
-    for (; i < ACTIVE_NOTIFS_MAX && actives[i].active; i ++);
+    for (; i < ACTIVE_NOTIFS_MAX && actives[i].active; ++i);
 
     if (i == ACTIVE_NOTIFS_MAX) {
         return -1; /* Full */
@@ -334,7 +347,7 @@ int play_source(uint32_t source, uint32_t buffer, bool looping)
 #elif BOX_NOTIFY
 void *do_playing(void *_p)
 {
-    (void)_p;
+    UNUSED_VAR(_p);
 
     while (true) {
         control_lock();
@@ -344,24 +357,16 @@ void *do_playing(void *_p)
             break;
         }
 
-        int i;
-
-        for (i = 0; i < ACTIVE_NOTIFS_MAX; i ++) {
+        for (size_t i = 0; i < ACTIVE_NOTIFS_MAX; ++i) {
             if (actives[i].box && time(NULL) >= actives[i].n_timeout) {
                 GError *ignore;
                 notify_notification_close(actives[i].box, &ignore);
-                actives[i].box = NULL;
-
-                if (actives[i].id_indicator) {
-                    *actives[i].id_indicator = -1;    /* reset indicator value */
-                }
-
-                memset(&actives[i], 0, sizeof(struct _ActiveNotifications));
+                clear_actives_index(i);
             }
         }
 
         control_unlock();
-        usleep(10000);
+        sleep_thread(10000L);
     }
 
     pthread_exit(NULL);
@@ -369,27 +374,48 @@ void *do_playing(void *_p)
 
 void graceful_clear(void)
 {
-    int i;
     control_lock();
 
-    for (i = 0; i < ACTIVE_NOTIFS_MAX; i ++) {
+    for (size_t i = 0; i < ACTIVE_NOTIFS_MAX; ++i) {
         if (actives[i].box) {
             GError *ignore;
             notify_notification_close(actives[i].box, &ignore);
-            actives[i].box = NULL;
         }
 
-        if (actives[i].id_indicator) {
-            *actives[i].id_indicator = -1;    /* reset indicator value */
-        }
-
-        memset(&actives[i], 0, sizeof(struct _ActiveNotifications));
+        clear_actives_index(i);
     }
 
     control_unlock();
 }
+
 #endif /* SOUND_NOTIFY */
 
+/* Kills all notifications for `id`. This must be called before freeing a ToxWindow. */
+void kill_notifs(int id)
+{
+    control_lock();
+
+    for (size_t i = 0; i < ACTIVE_NOTIFS_MAX; ++i) {
+        if (!actives[i].id_indicator) {
+            continue;
+        }
+
+        if (*actives[i].id_indicator == id) {
+#ifdef BOX_NOTIFY
+
+            if (actives[i].box) {
+                GError *ignore;
+                notify_notification_close(actives[i].box, &ignore);
+            }
+
+#endif // BOX_NOTIFY
+            clear_actives_index(i);
+        }
+    }
+
+    control_unlock();
+}
+
 /**********************************************************************************/
 /**********************************************************************************/
 /**********************************************************************************/
@@ -450,7 +476,7 @@ void terminate_notify(void)
 #ifdef SOUND_NOTIFY
     int i = 0;
 
-    for (; i < SOUNDS_SIZE; i ++) {
+    for (; i < SOUNDS_SIZE; ++i) {
         free(Control.sounds[i]);
     }
 
@@ -523,7 +549,6 @@ int play_notify_sound(Notification notif, uint64_t flags)
     return rc;
 }
 
-
 void stop_sound(int id)
 {
     if (id >= 0 && id < ACTIVE_NOTIFS_MAX && actives[id].looping && actives[id].active) {
@@ -536,15 +561,11 @@ void stop_sound(int id)
 
 #endif /* BOX_NOTIFY */
 
-        if (actives[id].id_indicator) {
-            *actives[id].id_indicator = -1;
-        }
-
         // alSourcei(actives[id].source, AL_LOOPING, false);
         alSourceStop(actives[id].source);
         alDeleteSources(1, &actives[id].source);
         alDeleteBuffers(1, &actives[id].buffer);
-        memset(&actives[id], 0, sizeof(struct _ActiveNotifications));
+        clear_actives_index(id);
     }
 }
 #endif /* SOUND_NOTIFY */
@@ -754,20 +775,18 @@ int box_notify2(ToxWindow *self, Notification notif, uint64_t flags, int id, con
     actives[id].size++;
     actives[id].n_timeout = get_unix_time() + Control.notif_timeout / 1000;
 
-    char formated[128 * 129] = {'\0'};
+    char *formatted = calloc(1, sizeof(char) * ((MAX_BOX_MSG_LEN + 1) * (MAX_BOX_MSG_LEN + 2)));
 
-    int i = 0;
-
-    for (; i < actives[id].size; i ++) {
-        strcat(formated, actives[id].messages[i]);
-        strcat(formated, "\n");
+    for (size_t i = 0; i < actives[id].size; ++i) {
+        strcat(formatted, actives[id].messages[i]);
+        strcat(formatted, "\n");
     }
 
-    formated[strlen(formated) - 1] = '\0';
-
-    notify_notification_update(actives[id].box, actives[id].title, formated, NULL);
+    notify_notification_update(actives[id].box, actives[id].title, formatted, NULL);
     notify_notification_show(actives[id].box, NULL);
 
+    free(formatted);
+
     control_unlock();
 
     return id;
@@ -863,20 +882,18 @@ int box_silent_notify2(ToxWindow *self, uint64_t flags, int id, const char *form
     actives[id].size ++;
     actives[id].n_timeout = get_unix_time() + Control.notif_timeout / 1000;
 
-    char formated[128 * 129] = {'\0'};
+    char *formatted = calloc(1, sizeof(char) * ((MAX_BOX_MSG_LEN + 1) * (MAX_BOX_MSG_LEN + 2)));
 
-    int i = 0;
-
-    for (; i < actives[id].size; i ++) {
-        strcat(formated, actives[id].messages[i]);
-        strcat(formated, "\n");
+    for (size_t i = 0; i < actives[id].size; ++i) {
+        strcat(formatted, actives[id].messages[i]);
+        strcat(formatted, "\n");
     }
 
-    formated[strlen(formated) - 1] = '\0';
-
-    notify_notification_update(actives[id].box, actives[id].title, formated, NULL);
+    notify_notification_update(actives[id].box, actives[id].title, formatted, NULL);
     notify_notification_show(actives[id].box, NULL);
 
+    free(formatted);
+
     control_unlock();
 
     return id;
