@@ -28,20 +28,20 @@
 #include <string.h>
 #include <wchar.h>
 
-#include "toxic.h"
-#include "windows.h"
-#include "prompt.h"
-#include "friendlist.h"
-#include "execute.h"
-#include "misc_tools.h"
-#include "toxic_strings.h"
-#include "log.h"
-#include "line_info.h"
-#include "settings.h"
-#include "input.h"
-#include "help.h"
-#include "notify.h"
 #include "autocomplete.h"
+#include "execute.h"
+#include "friendlist.h"
+#include "help.h"
+#include "input.h"
+#include "line_info.h"
+#include "log.h"
+#include "misc_tools.h"
+#include "notify.h"
+#include "prompt.h"
+#include "settings.h"
+#include "toxic.h"
+#include "toxic_strings.h"
+#include "windows.h"
 
 extern ToxWindow *prompt;
 extern struct user_settings *user_settings;
@@ -49,68 +49,47 @@ extern struct Winthread Winthread;
 
 extern FriendsList Friends;
 FriendRequests FrndRequests;
-#ifdef AUDIO
-#define AC_NUM_GLOB_COMMANDS_AUDIO 2
-#else
-#define AC_NUM_GLOB_COMMANDS_AUDIO 0
-#endif /* AUDIO */
-#ifdef VIDEO
-#define AC_NUM_GLOB_COMMANDS_VIDEO 2
-#else
-#define AC_NUM_GLOB_COMMANDS_VIDEO 0
-#endif /* VIDEO */
-#ifdef PYTHON
-#define AC_NUM_GLOB_COMMANDS_PYTHON 1
-#else
-#define AC_NUM_GLOB_COMMANDS_PYTHON 0
-#endif /* PYTHON */
-#ifdef QRCODE
-#define AC_NUM_GLOB_COMMANDS_QRCODE 1
-#else
-#define AC_NUM_GLOB_COMMANDS_QRCODE 0
-#endif /* QRCODE */
-#define AC_NUM_GLOB_COMMANDS (17 + AC_NUM_GLOB_COMMANDS_AUDIO + AC_NUM_GLOB_COMMANDS_VIDEO + AC_NUM_GLOB_COMMANDS_PYTHON + AC_NUM_GLOB_COMMANDS_QRCODE)
 
 /* Array of global command names used for tab completion. */
-static const char glob_cmd_list[AC_NUM_GLOB_COMMANDS][MAX_CMDNAME_SIZE] = {
-    { "/accept"     },
-    { "/add"        },
-    { "/avatar"     },
-    { "/clear"      },
-    { "/connect"    },
-    { "/decline"    },
-    { "/exit"       },
-    { "/group"      },
-    { "/help"       },
-    { "/log"        },
-    { "/myid"       },
+static const char *glob_cmd_list[] = {
+    "/accept",
+    "/add",
+    "/avatar",
+    "/clear",
+    "/connect",
+    "/decline",
+    "/exit",
+    "/conference",
+    "/help",
+    "/log",
+    "/myid",
 #ifdef QRCODE
-    { "/myqr"       },
+    "/myqr",
 #endif /* QRCODE */
-    { "/nick"       },
-    { "/note"       },
-    { "/nospam"     },
-    { "/quit"       },
-    { "/requests"   },
-    { "/status"     },
+    "/nick",
+    "/note",
+    "/nospam",
+    "/quit",
+    "/requests",
+    "/status",
 
 #ifdef AUDIO
 
-    { "/lsdev"       },
-    { "/sdev"        },
+    "/lsdev",
+    "/sdev",
 
 #endif /* AUDIO */
 
 #ifdef VIDEO
 
-    { "/lsvdev"      },
-    { "/svdev"       },
+    "/lsvdev",
+    "/svdev",
 
 #endif /* VIDEO */
 
 #ifdef PYTHON
 
-    { "/run"         },
+    "/run",
 
 #endif /* PYTHON */
 
@@ -139,6 +118,8 @@ void kill_prompt_window(ToxWindow *self)
 /* callback: Updates own connection status in prompt statusbar */
 void on_self_connection_status(Tox *m, Tox_Connection connection_status, void *userdata)
 {
+    UNUSED_VAR(m);
+    UNUSED_VAR(userdata);
     StatusBar *statusbar = prompt->stb;
     statusbar->connection = connection_status;
 }
@@ -163,7 +144,7 @@ void prompt_update_statusmessage(ToxWindow *prompt, Tox *m, const char *statusms
     tox_self_set_status_message(m, (const uint8_t *) statusmsg, len, &err);
 
     if (err != TOX_ERR_SET_INFO_OK) {
-        line_info_add(prompt, NULL, NULL, NULL, SYS_MSG, 0, 0, "Failed to set note (error %d)\n", err);
+        line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to set note (error %d)\n", err);
     }
 }
 
@@ -210,7 +191,10 @@ static int add_friend_request(const char *public_key, const char *data)
     return -1;
 }
 
-static void prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
+/*
+ * Return true if input is recognized by handler
+ */
+static bool prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
 {
     ChatContext *ctx = self->chatwin;
 
@@ -218,8 +202,10 @@ static void prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
     getyx(self->window, y, x);
     getmaxyx(self->window, y2, x2);
 
+    UNUSED_VAR(y);
+
     if (x2 <= 0 || y2 <= 0) {
-        return;
+        return false;
     }
 
     if (ctx->pastemode && key == '\r') {
@@ -229,21 +215,27 @@ static void prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
     /* ignore non-menu related input if active */
     if (self->help->active) {
         help_onKey(self, key);
-        return;
+        return true;
     }
 
     if (ltr || key == '\n') {    /* char is printable */
-        input_new_char(self, key, x, y, x2, y2);
-        return;
+        input_new_char(self, key, x, x2);
+        return true;
     }
 
     if (line_info_onKey(self, key)) {
-        return;
+        return true;
     }
 
-    input_handle(self, key, x, y, x2, y2);
+    if (input_handle(self, key, x, x2)) {
+        return true;
+    }
+
+    int input_ret = false;
 
     if (key == '\t') {    /* TAB key: auto-completes command */
+        input_ret = true;
+
         if (ctx->len > 1 && ctx->line[0] == '/') {
             int diff = -1;
 
@@ -259,14 +251,14 @@ static void prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
 #endif
 
             else if (wcsncmp(ctx->line, L"/status ", wcslen(L"/status ")) == 0) {
-                const char status_cmd_list[3][8] = {
-                    {"online"},
-                    {"away"},
-                    {"busy"},
+                const char *status_cmd_list[] = {
+                    "online",
+                    "away",
+                    "busy",
                 };
-                diff = complete_line(self, status_cmd_list, 3, 8);
+                diff = complete_line(self, status_cmd_list, sizeof(status_cmd_list) / sizeof(char *));
             } else {
-                diff = complete_line(self, glob_cmd_list, AC_NUM_GLOB_COMMANDS, MAX_CMDNAME_SIZE);
+                diff = complete_line(self, glob_cmd_list, sizeof(glob_cmd_list) / sizeof(char *));
             }
 
             if (diff != -1) {
@@ -281,31 +273,36 @@ static void prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
             sound_notify(self, notif_error, 0, NULL);
         }
     } else if (key == '\r') {
+        input_ret = true;
+
         rm_trailing_spaces_buf(ctx);
 
         if (!wstring_is_empty(ctx->line)) {
             add_line_to_hist(ctx);
             wstrsubst(ctx->line, L'¶', L'\n');
 
-            char line[MAX_STR_SIZE] = {0};
+            char line[MAX_STR_SIZE];
 
             if (wcs_to_mbs_buf(line, ctx->line, MAX_STR_SIZE) == -1) {
-                memset(&line, 0, sizeof(line));
+                line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED, " * Failed to parse message.");
+            } else {
+                line_info_add(self, false, NULL, NULL, PROMPT, 0, 0, "%s", line);
+                execute(ctx->history, self, m, line, GLOBAL_COMMAND_MODE);
             }
-
-            line_info_add(self, NULL, NULL, NULL, PROMPT, 0, 0, "%s", line);
-            execute(ctx->history, self, m, line, GLOBAL_COMMAND_MODE);
         }
 
         wclear(ctx->linewin);
-        wmove(self->window, y2 - CURS_Y_OFFSET, 0);
+        wmove(self->window, y2, 0);
         reset_buf(ctx);
     }
+
+    return input_ret;
 }
 
 static void prompt_onDraw(ToxWindow *self, Tox *m)
 {
-    int x2, y2;
+    int x2;
+    int y2;
     getmaxyx(self->window, y2, x2);
 
     if (y2 <= 0 || x2 <= 0) {
@@ -320,64 +317,88 @@ static void prompt_onDraw(ToxWindow *self, Tox *m)
 
     wclear(ctx->linewin);
 
-    curs_set(1);
-
     if (ctx->len > 0) {
-        mvwprintw(ctx->linewin, 1, 0, "%ls", &ctx->line[ctx->start]);
+        mvwprintw(ctx->linewin, 0, 0, "%ls", &ctx->line[ctx->start]);
     }
 
+    mvwhline(ctx->linewin, 0, ctx->len, ' ', x2 - ctx->len);
+
+    curs_set(1);
+
     StatusBar *statusbar = self->stb;
 
-    mvwhline(statusbar->topline, 1, 0, ACS_HLINE, x2);
     wmove(statusbar->topline, 0, 0);
 
     pthread_mutex_lock(&Winthread.lock);
     Tox_Connection connection = statusbar->connection;
+    Tox_User_Status status = statusbar->status;
     pthread_mutex_unlock(&Winthread.lock);
 
     if (connection != TOX_CONNECTION_NONE) {
         int colour = MAGENTA;
         const char *status_text = "ERROR";
 
-        pthread_mutex_lock(&Winthread.lock);
-        Tox_User_Status status = statusbar->status;
-        pthread_mutex_unlock(&Winthread.lock);
-
         switch (status) {
             case TOX_USER_STATUS_NONE:
                 status_text = "Online";
-                colour = GREEN;
+                colour = STATUS_ONLINE;
                 break;
 
             case TOX_USER_STATUS_AWAY:
                 status_text = "Away";
-                colour = YELLOW;
+                colour = STATUS_AWAY;
                 break;
 
             case TOX_USER_STATUS_BUSY:
                 status_text = "Busy";
-                colour = RED;
+                colour = STATUS_BUSY;
                 break;
         }
 
-        wattron(statusbar->topline, COLOR_PAIR(colour) | A_BOLD);
-        wprintw(statusbar->topline, " [%s]", status_text);
-        wattroff(statusbar->topline, COLOR_PAIR(colour) | A_BOLD);
+        wattron(statusbar->topline, COLOR_PAIR(BAR_ACCENT));
+        wprintw(statusbar->topline, " [");
+        wattroff(statusbar->topline, COLOR_PAIR(BAR_ACCENT));
+
+        wattron(statusbar->topline, A_BOLD | COLOR_PAIR(colour));
+        wprintw(statusbar->topline, "%s", status_text);
+        wattroff(statusbar->topline, A_BOLD | COLOR_PAIR(colour));
+
+        wattron(statusbar->topline, COLOR_PAIR(BAR_ACCENT));
+        wprintw(statusbar->topline, "]");
+        wattroff(statusbar->topline, COLOR_PAIR(BAR_ACCENT));
+
+        wattron(statusbar->topline, COLOR_PAIR(BAR_TEXT));
 
-        wattron(statusbar->topline, A_BOLD);
         pthread_mutex_lock(&Winthread.lock);
         wprintw(statusbar->topline, " %s", statusbar->nick);
         pthread_mutex_unlock(&Winthread.lock);
-        wattroff(statusbar->topline, A_BOLD);
     } else {
-        wprintw(statusbar->topline, " [Offline]");
-        wattron(statusbar->topline, A_BOLD);
+        wattron(statusbar->topline, COLOR_PAIR(BAR_ACCENT));
+        wprintw(statusbar->topline, " [");
+        wattroff(statusbar->topline, COLOR_PAIR(BAR_ACCENT));
+
+        wattron(statusbar->topline, COLOR_PAIR(BAR_TEXT));
+        wprintw(statusbar->topline, "Offline");
+        wattroff(statusbar->topline, COLOR_PAIR(BAR_TEXT));
+
+        wattron(statusbar->topline, COLOR_PAIR(BAR_ACCENT));
+        wprintw(statusbar->topline, "]");
+        wattroff(statusbar->topline, COLOR_PAIR(BAR_ACCENT));
+
+        wattron(statusbar->topline, COLOR_PAIR(BAR_TEXT));
+
         pthread_mutex_lock(&Winthread.lock);
         wprintw(statusbar->topline, " %s", statusbar->nick);
         pthread_mutex_unlock(&Winthread.lock);
-        wattroff(statusbar->topline, A_BOLD);
     }
 
+    int s_y;
+    int s_x;
+    getyx(statusbar->topline, s_y, s_x);
+
+    mvwhline(statusbar->topline, s_y, s_x, ' ', x2 - s_x);
+    wattroff(statusbar->topline, COLOR_PAIR(BAR_TEXT));
+
     /* Reset statusbar->statusmsg on window resize */
     if (x2 != self->x) {
         char statusmsg[TOX_MAX_STATUS_MESSAGE_LENGTH];
@@ -405,19 +426,27 @@ static void prompt_onDraw(ToxWindow *self, Tox *m)
     }
 
     if (statusbar->statusmsg[0]) {
-        wprintw(statusbar->topline, " : %s", statusbar->statusmsg);
+        wattron(statusbar->topline, COLOR_PAIR(BAR_ACCENT));
+        wprintw(statusbar->topline, " | ");
+        wattroff(statusbar->topline, COLOR_PAIR(BAR_ACCENT));
+
+        wattron(statusbar->topline, COLOR_PAIR(BAR_TEXT));
+        wprintw(statusbar->topline, "%s", statusbar->statusmsg);
+        wattroff(statusbar->topline, COLOR_PAIR(BAR_TEXT));
     }
 
     pthread_mutex_unlock(&Winthread.lock);
 
-    mvwhline(self->window, y2 - CHATBOX_HEIGHT, 0, ACS_HLINE, x2);
-
-    int y, x;
+    int y;
+    int x;
     getyx(self->window, y, x);
-    (void) x;
+
+    UNUSED_VAR(x);
 
     int new_x = ctx->start ? x2 - 1 : MAX(0, wcswidth(ctx->line, ctx->pos));
-    wmove(self->window, y + 1, new_x);
+    wmove(self->window, y, new_x);
+
+    draw_window_bar(self);
 
     wnoutrefresh(self->window);
 
@@ -437,8 +466,6 @@ static void prompt_onConnectionChange(ToxWindow *self, Tox *m, uint32_t friendnu
         snprintf(nick, sizeof(nick), "%s", UNKNOWN_NAME);
     }
 
-    char timefrmt[TIME_STR_SIZE];
-    get_time_str(timefrmt, sizeof(timefrmt));
     const char *msg;
 
     if (user_settings->show_connection_msg == SHOW_WELCOME_MSG_OFF) {
@@ -447,7 +474,7 @@ static void prompt_onConnectionChange(ToxWindow *self, Tox *m, uint32_t friendnu
 
     if (connection_status != TOX_CONNECTION_NONE && Friends.list[friendnum].connection_status == TOX_CONNECTION_NONE) {
         msg = "has come online";
-        line_info_add(self, timefrmt, nick, NULL, CONNECTION, 0, GREEN, msg);
+        line_info_add(self, true, nick, NULL, CONNECTION, 0, GREEN, msg);
         write_to_log(msg, nick, ctx->log, true);
 
         if (self->active_box != -1) {
@@ -459,7 +486,7 @@ static void prompt_onConnectionChange(ToxWindow *self, Tox *m, uint32_t friendnu
         }
     } else if (connection_status == TOX_CONNECTION_NONE) {
         msg = "has gone offline";
-        line_info_add(self, timefrmt, nick, NULL, DISCONNECTION, 0, RED, msg);
+        line_info_add(self, true, nick, NULL, DISCONNECTION, 0, RED, msg);
         write_to_log(msg, nick, ctx->log, true);
 
         if (self->active_box != -1) {
@@ -474,23 +501,23 @@ static void prompt_onConnectionChange(ToxWindow *self, Tox *m, uint32_t friendnu
 
 static void prompt_onFriendRequest(ToxWindow *self, Tox *m, const char *key, const char *data, size_t length)
 {
+    UNUSED_VAR(m);
+    UNUSED_VAR(length);
+
     ChatContext *ctx = self->chatwin;
 
-    char timefrmt[TIME_STR_SIZE];
-    get_time_str(timefrmt, sizeof(timefrmt));
-
-    line_info_add(self, timefrmt, NULL, NULL, SYS_MSG, 0, 0, "Friend request with the message '%s'", data);
+    line_info_add(self, true, NULL, NULL, SYS_MSG, 0, 0, "Friend request with the message '%s'", data);
     write_to_log("Friend request with the message '%s'", "", ctx->log, true);
 
     int n = add_friend_request(key, data);
 
     if (n == -1) {
         const char *errmsg = "Friend request queue is full. Discarding request.";
-        line_info_add(self, NULL, NULL, NULL, SYS_MSG, 0, 0, errmsg);
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, errmsg);
         return;
     }
 
-    line_info_add(self, NULL, NULL, NULL, SYS_MSG, 0, 0, "Type \"/accept %d\" or \"/decline %d\"", n, n);
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Type \"/accept %d\" or \"/decline %d\"", n, n);
     sound_notify(self, generic_message, NT_WNDALERT_1 | NT_NOTIFWND, NULL);
 }
 
@@ -503,7 +530,7 @@ void prompt_init_statusbar(ToxWindow *self, Tox *m, bool first_time_run)
         exit_toxic_err("failed in prompt_init_statusbar", FATALERR_CURSES);
     }
 
-    (void) y2;
+    UNUSED_VAR(y2);
 
     /* Init statusbar info */
     StatusBar *statusbar = self->stb;
@@ -535,29 +562,50 @@ void prompt_init_statusbar(ToxWindow *self, Tox *m, bool first_time_run)
     prompt_update_nick(prompt, nick);
 
     /* Init statusbar subwindow */
-    statusbar->topline = subwin(self->window, 2, x2, 0, 0);
+    statusbar->topline = subwin(self->window, TOP_BAR_HEIGHT, x2, 0, 0);
 }
 
 static void print_welcome_msg(ToxWindow *self)
 {
-    line_info_add(self, NULL, NULL, NULL, SYS_MSG, 1, BLUE, "    _____ _____  _____ ____ ");
-    line_info_add(self, NULL, NULL, NULL, SYS_MSG, 1, BLUE, "   |_   _/ _ \\ \\/ /_ _/ ___|");
-    line_info_add(self, NULL, NULL, NULL, SYS_MSG, 1, BLUE, "     | || | | \\  / | | |    ");
-    line_info_add(self, NULL, NULL, NULL, SYS_MSG, 1, BLUE, "     | || |_| /  \\ | | |___ ");
-    line_info_add(self, NULL, NULL, NULL, SYS_MSG, 1, BLUE, "     |_| \\___/_/\\_\\___\\____| v." TOXICVER);
-    line_info_add(self, NULL, NULL, NULL, SYS_MSG, 0, 0, "");
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, BLUE, "    _____ _____  _____ ____ ");
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, BLUE, "   |_   _/ _ \\ \\/ /_ _/ ___|");
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, BLUE, "     | || | | \\  / | | |    ");
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, BLUE, "     | || |_| /  \\ | | |___ ");
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, BLUE, "     |_| \\___/_/\\_\\___\\____| v." TOXICVER);
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "");
 
-    const char *msg = "Welcome to Toxic, a free, open source Tox-based instant messenging client.";
-    line_info_add(self, NULL, NULL, NULL, SYS_MSG, 1, CYAN, msg);
+    const char *msg = "Welcome to Toxic, a free, open source Tox-based instant messaging client.";
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, CYAN, msg);
     msg = "Type \"/help\" for assistance. Further help may be found via the man page.";
-    line_info_add(self, NULL, NULL, NULL, SYS_MSG, 1, CYAN, msg);
-    line_info_add(self, NULL, NULL, NULL, SYS_MSG, 0, 0, "");
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, CYAN, msg);
+    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "");
+}
+
+static void prompt_init_log(ToxWindow *self, Tox *m, const char *self_name)
+{
+    ChatContext *ctx = self->chatwin;
+
+    char myid[TOX_ADDRESS_SIZE];
+    tox_self_get_address(m, (uint8_t *) myid);
+
+    if (log_init(ctx->log, self->name, myid, NULL, LOG_TYPE_PROMPT) != 0) {
+        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Warning: Log failed to initialize.");
+        return;
+    }
+
+    if (user_settings->autolog == AUTOLOG_ON) {
+        if (log_enable(ctx->log) == -1) {
+            line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Warning: Failed to enable log.");
+        }
+    }
 }
 
 static void prompt_onInit(ToxWindow *self, Tox *m)
 {
     curs_set(1);
-    int y2, x2;
+
+    int y2;
+    int x2;
     getmaxyx(self->window, y2, x2);
 
     if (y2 <= 0 || x2 <= 0) {
@@ -565,8 +613,10 @@ static void prompt_onInit(ToxWindow *self, Tox *m)
     }
 
     ChatContext *ctx = self->chatwin;
-    ctx->history = subwin(self->window, y2 - CHATBOX_HEIGHT + 1, x2, 0, 0);
-    ctx->linewin = subwin(self->window, CHATBOX_HEIGHT, x2, y2 - CHATBOX_HEIGHT, 0);
+
+    ctx->history = subwin(self->window, y2 - CHATBOX_HEIGHT - WINDOW_BAR_HEIGHT, x2, 0, 0);
+    self->window_bar = subwin(self->window, WINDOW_BAR_HEIGHT, x2, y2 - (CHATBOX_HEIGHT + WINDOW_BAR_HEIGHT), 0);
+    ctx->linewin = subwin(self->window, CHATBOX_HEIGHT, x2, y2 - WINDOW_BAR_HEIGHT, 0);
 
     ctx->log = calloc(1, sizeof(struct chatlog));
     ctx->hst = calloc(1, sizeof(struct history));
@@ -577,14 +627,7 @@ static void prompt_onInit(ToxWindow *self, Tox *m)
 
     line_info_init(ctx->hst);
 
-    if (user_settings->autolog == AUTOLOG_ON) {
-        char myid[TOX_ADDRESS_SIZE];
-        tox_self_get_address(m, (uint8_t *) myid);
-
-        if (log_enable(self->name, myid, NULL, ctx->log, LOG_PROMPT) == -1) {
-            line_info_add(self, NULL, NULL, NULL, SYS_MSG, 0, 0, "Warning: Log failed to initialize.");
-        }
-    }
+    prompt_init_log(self, m, self->name);
 
     scrollok(ctx->history, 0);
     wmove(self->window, y2 - CURS_Y_OFFSET, 0);
@@ -603,7 +646,7 @@ ToxWindow *new_prompt(void)
     }
 
     ret->num = -1;
-    ret->is_prompt = true;
+    ret->type = WINDOW_TYPE_PROMPT;
 
     ret->onKey = &prompt_onKey;
     ret->onDraw = &prompt_onDraw;
@@ -611,7 +654,7 @@ ToxWindow *new_prompt(void)
     ret->onConnectionChange = &prompt_onConnectionChange;
     ret->onFriendRequest = &prompt_onFriendRequest;
 
-    strcpy(ret->name, "home");
+    strcpy(ret->name, "Home");
 
     ChatContext *chatwin = calloc(1, sizeof(ChatContext));
     StatusBar *stb = calloc(1, sizeof(StatusBar));
