@@ -20,53 +20,53 @@
  *
  */
 
+#include <ctype.h>
 #include <curses.h>
 #include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <getopt.h>
+#include <limits.h>
+#include <locale.h>
+#include <netdb.h>
+#include <pthread.h>
+#include <signal.h>
+#include <stdarg.h>
 #include <stdbool.h>
 #include <stdint.h>
-#include <stdarg.h>
-#include <signal.h>
-#include <locale.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
-#include <time.h>
-#include <pthread.h>
-#include <getopt.h>
-#include <netdb.h>
+#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <sys/socket.h>
-#include <unistd.h>
-#include <limits.h>
 #include <termios.h>
-#include <ctype.h>
+#include <time.h>
+#include <unistd.h>
 
 #include <curl/curl.h>
-#include <tox/tox.h>
 #include <tox/toxencryptsave.h>
+#include <tox/tox.h>
 
+#include "audio_device.h"
+#include "bootstrap.h"
+#include "conference.h"
 #include "configdir.h"
+#include "execute.h"
+#include "file_transfers.h"
+#include "friendlist.h"
+#include "line_info.h"
+#include "log.h"
+#include "message_queue.h"
+#include "misc_tools.h"
+#include "name_lookup.h"
+#include "notify.h"
+#include "prompt.h"
+#include "settings.h"
+#include "term_mplex.h"
 #include "toxic.h"
 #include "windows.h"
-#include "friendlist.h"
-#include "prompt.h"
-#include "misc_tools.h"
-#include "groupchat.h"
-#include "file_transfers.h"
-#include "line_info.h"
-#include "settings.h"
-#include "log.h"
-#include "notify.h"
-#include "audio_device.h"
-#include "message_queue.h"
-#include "execute.h"
-#include "term_mplex.h"
-#include "name_lookup.h"
-#include "bootstrap.h"
 
 #ifdef X11
-#include "xtra.h"
+#include "x11focus.h"
 #endif
 
 #ifdef AUDIO
@@ -94,7 +94,6 @@ ToxWindow *prompt = NULL;
 #define DATANAME  "toxic_profile.tox"
 #define BLOCKNAME "toxic_blocklist"
 
-#define AUTOSAVE_FREQ 600
 #define MIN_PASSWORD_LEN 6
 #define MAX_PASSWORD_LEN 64
 
@@ -104,7 +103,6 @@ struct av_thread av_thread;
 struct arg_opts arg_opts;
 struct user_settings *user_settings = NULL;
 
-
 static struct user_password {
     bool data_is_encrypted;
     char pass[MAX_PASSWORD_LEN + 1];
@@ -115,6 +113,8 @@ static time_t last_signal_time;
 
 static void catch_SIGINT(int sig)
 {
+    UNUSED_VAR(sig);
+
     time_t cur_time = get_unix_time();
 
     if (difftime(cur_time, last_signal_time) <= 1) {
@@ -126,14 +126,21 @@ static void catch_SIGINT(int sig)
 
 static void catch_SIGSEGV(int sig)
 {
-    freopen("/dev/tty", "w", stderr);    // make sure stderr is enabled since we may have disabled it
+    UNUSED_VAR(sig);
+
+    if (freopen("/dev/tty", "w", stderr)) {    // make sure stderr is enabled since we may have disabled it
+        fprintf(stderr, "Caught SIGSEGV: Aborting toxic session.\n");
+    }
+
     endwin();
-    fprintf(stderr, "Caught SIGSEGV: Aborting toxic session.\n");
+
     exit(EXIT_FAILURE);
 }
 
 static void flag_window_resize(int sig)
 {
+    UNUSED_VAR(sig);
+
     Winthread.flag_resize = 1;
 }
 
@@ -165,10 +172,15 @@ void free_global_data(void)
 void exit_toxic_success(Tox *m)
 {
     store_data(m, DATA_FILE);
-    memset(&user_password, 0, sizeof(struct user_password));
+
+    user_password = (struct user_password) {
+        0
+    };
+
+    terminate_notify();
+
     kill_all_file_transfers(m);
     kill_all_windows(m);
-    terminate_notify();
 
 #ifdef AUDIO
 #ifdef VIDEO
@@ -183,14 +195,17 @@ void exit_toxic_success(Tox *m)
 
     free_global_data();
     tox_kill(m);
+
+    if (arg_opts.log_fp != NULL) {
+        fclose(arg_opts.log_fp);
+        arg_opts.log_fp = NULL;
+    }
+
     endwin();
     curl_global_cleanup();
 
 #ifdef X11
-    /* We have to terminate xtra last coz reasons
-     * Please don't call this anywhere else coz trust me
-     */
-    terminate_xtra();
+    terminate_x11focus();
 #endif /* X11 */
 
     exit(EXIT_SUCCESS);
@@ -199,12 +214,31 @@ void exit_toxic_success(Tox *m)
 void exit_toxic_err(const char *errmsg, int errcode)
 {
     free_global_data();
-    freopen("/dev/tty", "w", stderr);
+
+    if (freopen("/dev/tty", "w", stderr)) {
+        fprintf(stderr, "Toxic session aborted with error code %d (%s)\n", errcode, errmsg);
+    }
+
     endwin();
-    fprintf(stderr, "Toxic session aborted with error code %d (%s)\n", errcode, errmsg);
     exit(EXIT_FAILURE);
 }
 
+void cb_toxcore_logger(Tox *m, TOX_LOG_LEVEL level, const char *file, uint32_t line, const char *func,
+                       const char *message, void *user_data)
+{
+    UNUSED_VAR(user_data);
+    UNUSED_VAR(file);
+    UNUSED_VAR(m);
+
+    if (user_data) {
+        FILE *fp = (FILE *)user_data;
+        fprintf(fp, "[%d] %u:%s() - %s\n", level, line, func, message);
+        fflush(fp);
+    } else {
+        fprintf(stderr, "[%d] %u:%s() - %s\n", level, line, func, message);
+    }
+}
+
 static void init_term(void)
 {
 #if HAVE_WIDECHAR
@@ -222,10 +256,14 @@ static void init_term(void)
     keypad(stdscr, 1);
     noecho();
     nonl();
-    timeout(100);
+    timeout(50);
 
     if (has_colors()) {
         short bg_color = COLOR_BLACK;
+        short bar_bg_color = COLOR_BLUE;
+        short bar_fg_color = COLOR_WHITE;
+        short bar_accent_color = COLOR_CYAN;
+        short bar_notify_color = COLOR_YELLOW;
         start_color();
 
         if (user_settings->colour_theme == NATIVE_COLS) {
@@ -234,15 +272,120 @@ static void init_term(void)
             }
         }
 
-        init_pair(0, COLOR_WHITE, COLOR_BLACK);
-        init_pair(1, COLOR_GREEN, bg_color);
-        init_pair(2, COLOR_CYAN, bg_color);
-        init_pair(3, COLOR_RED, bg_color);
-        init_pair(4, COLOR_BLUE, bg_color);
-        init_pair(5, COLOR_YELLOW, bg_color);
-        init_pair(6, COLOR_MAGENTA, bg_color);
-        init_pair(7, COLOR_BLACK, COLOR_BLACK);
-        init_pair(8, COLOR_BLACK, COLOR_WHITE);
+        if (!string_is_empty(user_settings->color_bar_bg)) {
+            if (strcmp(user_settings->color_bar_bg, "black") == 0) {
+                bar_bg_color = COLOR_BLACK;
+            } else if (strcmp(user_settings->color_bar_bg, "red") == 0) {
+                bar_bg_color = COLOR_RED;
+            } else if (strcmp(user_settings->color_bar_bg, "blue") == 0) {
+                bar_bg_color = COLOR_BLUE;
+            } else if (strcmp(user_settings->color_bar_bg, "cyan") == 0) {
+                bar_bg_color = COLOR_CYAN;
+            } else if (strcmp(user_settings->color_bar_bg, "green") == 0) {
+                bar_bg_color = COLOR_GREEN;
+            } else if (strcmp(user_settings->color_bar_bg, "yellow") == 0) {
+                bar_bg_color = COLOR_YELLOW;
+            } else if (strcmp(user_settings->color_bar_bg, "magenta") == 0) {
+                bar_bg_color = COLOR_MAGENTA;
+            } else if (strcmp(user_settings->color_bar_bg, "white") == 0) {
+                bar_bg_color = COLOR_WHITE;
+            } else {
+                bar_bg_color = COLOR_BLUE;
+            }
+        } else {
+            bar_bg_color = COLOR_BLUE;
+        }
+
+        if (!string_is_empty(user_settings->color_bar_fg)) {
+            if (strcmp(user_settings->color_bar_fg, "black") == 0) {
+                bar_fg_color = COLOR_BLACK;
+            } else if (strcmp(user_settings->color_bar_fg, "red") == 0) {
+                bar_fg_color = COLOR_RED;
+            } else if (strcmp(user_settings->color_bar_fg, "blue") == 0) {
+                bar_fg_color = COLOR_BLUE;
+            } else if (strcmp(user_settings->color_bar_fg, "cyan") == 0) {
+                bar_fg_color = COLOR_CYAN;
+            } else if (strcmp(user_settings->color_bar_fg, "green") == 0) {
+                bar_fg_color = COLOR_GREEN;
+            } else if (strcmp(user_settings->color_bar_fg, "yellow") == 0) {
+                bar_fg_color = COLOR_YELLOW;
+            } else if (strcmp(user_settings->color_bar_fg, "magenta") == 0) {
+                bar_fg_color = COLOR_MAGENTA;
+            } else if (strcmp(user_settings->color_bar_fg, "white") == 0) {
+                bar_fg_color = COLOR_WHITE;
+            } else {
+                bar_fg_color = COLOR_WHITE;
+            }
+        } else {
+            bar_fg_color = COLOR_WHITE;
+        }
+
+        if (!string_is_empty(user_settings->color_bar_accent)) {
+            if (strcmp(user_settings->color_bar_accent, "black") == 0) {
+                bar_accent_color = COLOR_BLACK;
+            } else if (strcmp(user_settings->color_bar_accent, "red") == 0) {
+                bar_accent_color = COLOR_RED;
+            } else if (strcmp(user_settings->color_bar_accent, "blue") == 0) {
+                bar_accent_color = COLOR_BLUE;
+            } else if (strcmp(user_settings->color_bar_accent, "cyan") == 0) {
+                bar_accent_color = COLOR_CYAN;
+            } else if (strcmp(user_settings->color_bar_accent, "green") == 0) {
+                bar_accent_color = COLOR_GREEN;
+            } else if (strcmp(user_settings->color_bar_accent, "yellow") == 0) {
+                bar_accent_color = COLOR_YELLOW;
+            } else if (strcmp(user_settings->color_bar_accent, "magenta") == 0) {
+                bar_accent_color = COLOR_MAGENTA;
+            } else if (strcmp(user_settings->color_bar_accent, "white") == 0) {
+                bar_accent_color = COLOR_WHITE;
+            } else {
+                bar_accent_color = COLOR_CYAN;
+            }
+        } else {
+            bar_accent_color = COLOR_CYAN;
+        }
+
+        if (!string_is_empty(user_settings->color_bar_notify)) {
+            if (strcmp(user_settings->color_bar_notify, "black") == 0) {
+                bar_notify_color = COLOR_BLACK;
+            } else if (strcmp(user_settings->color_bar_notify, "red") == 0) {
+                bar_notify_color = COLOR_RED;
+            } else if (strcmp(user_settings->color_bar_notify, "blue") == 0) {
+                bar_notify_color = COLOR_BLUE;
+            } else if (strcmp(user_settings->color_bar_notify, "cyan") == 0) {
+                bar_notify_color = COLOR_CYAN;
+            } else if (strcmp(user_settings->color_bar_notify, "green") == 0) {
+                bar_notify_color = COLOR_GREEN;
+            } else if (strcmp(user_settings->color_bar_notify, "yellow") == 0) {
+                bar_notify_color = COLOR_YELLOW;
+            } else if (strcmp(user_settings->color_bar_notify, "magenta") == 0) {
+                bar_notify_color = COLOR_MAGENTA;
+            } else if (strcmp(user_settings->color_bar_notify, "white") == 0) {
+                bar_notify_color = COLOR_WHITE;
+            } else {
+                bar_notify_color = COLOR_YELLOW;
+            }
+        } else {
+            bar_notify_color = COLOR_YELLOW;
+        }
+
+        init_pair(WHITE, COLOR_WHITE, COLOR_BLACK);
+        init_pair(GREEN, COLOR_GREEN, bg_color);
+        init_pair(CYAN, COLOR_CYAN, bg_color);
+        init_pair(RED, COLOR_RED, bg_color);
+        init_pair(BLUE, COLOR_BLUE, bg_color);
+        init_pair(YELLOW, COLOR_YELLOW, bg_color);
+        init_pair(MAGENTA, COLOR_MAGENTA, bg_color);
+        init_pair(BLACK, COLOR_BLACK, COLOR_BLACK);
+        init_pair(BLUE_BLACK, COLOR_BLUE, COLOR_BLACK);
+        init_pair(BLACK_WHITE, COLOR_BLACK, COLOR_WHITE);
+        init_pair(BLACK_BG, COLOR_BLACK, bar_bg_color);
+        init_pair(PURPLE_BG, COLOR_MAGENTA, bar_bg_color);
+        init_pair(BAR_TEXT, bar_fg_color, bar_bg_color);
+        init_pair(BAR_ACCENT, bar_accent_color, bar_bg_color);
+        init_pair(BAR_NOTIFY, bar_notify_color, bar_bg_color);
+        init_pair(STATUS_ONLINE, COLOR_GREEN, bar_bg_color);
+        init_pair(STATUS_AWAY, COLOR_YELLOW, bar_bg_color);
+        init_pair(STATUS_BUSY, COLOR_RED, bar_bg_color);
     }
 
     refresh();
@@ -319,37 +462,74 @@ static void load_friendlist(Tox *m)
     sort_friendlist_index();
 }
 
-static void load_groups(ToxWindow *prompt, Tox *m)
+static void load_conferences(Tox *m)
 {
-    size_t i;
     size_t num_chats = tox_conference_get_chatlist_size(m);
-    uint32_t chatlist[num_chats];
 
-    if (num_chats) {
-        tox_conference_get_chatlist(m, chatlist);
+    if (num_chats == 0) {
+        return;
     }
 
-    for (i = 0; i < num_chats; ++i) {
-        uint32_t groupnum = chatlist[i];
+    uint32_t *chatlist = malloc(num_chats * sizeof(uint32_t));
+
+    if (chatlist == NULL) {
+        fprintf(stderr, "malloc() failed in load_conferences()\n");
+        return;
+    }
+
+    tox_conference_get_chatlist(m, chatlist);
+
+    for (size_t i = 0; i < num_chats; ++i) {
+        uint32_t conferencenum = chatlist[i];
 
         if (get_num_active_windows() >= MAX_WINDOWS_NUM) {
-            tox_conference_delete(m, groupnum, NULL);
+            tox_conference_delete(m, conferencenum, NULL);
             continue;
         }
 
         Tox_Err_Conference_Get_Type err;
-        Tox_Conference_Type type = tox_conference_get_type(m, groupnum, &err);
+        Tox_Conference_Type type = tox_conference_get_type(m, conferencenum, &err);
 
         if (err != TOX_ERR_CONFERENCE_GET_TYPE_OK) {
-            tox_conference_delete(m, groupnum, NULL);
+            tox_conference_delete(m, conferencenum, NULL);
             continue;
         }
 
-        if (init_groupchat_win(prompt, m, groupnum, type) == -1) {
-            tox_conference_delete(m, groupnum, NULL);
+        Tox_Err_Conference_Title t_err;
+        size_t length = tox_conference_get_title_size(m, conferencenum, &t_err);
+        uint8_t title[MAX_STR_SIZE];
+
+        if (t_err != TOX_ERR_CONFERENCE_TITLE_OK || length >= sizeof(title)) {
+            length = 0;
+        } else {
+            tox_conference_get_title(m, conferencenum, title, &t_err);
+
+            if (t_err != TOX_ERR_CONFERENCE_TITLE_OK) {
+                length = 0;
+            }
+        }
+
+        title[length] = 0;
+
+        int win_idx = init_conference_win(m, conferencenum, type, (const char *) title, length);
+
+        if (win_idx == -1) {
+            tox_conference_delete(m, conferencenum, NULL);
             continue;
         }
+
+        if (type == TOX_CONFERENCE_TYPE_AV) {
+            line_info_add(get_window_ptr(win_idx), NULL, NULL, NULL, SYS_MSG, 0, 0,
+#ifdef AUDIO
+                          "Use \"/audio on\" to enable audio in this conference."
+#else
+                          "Audio support disabled by compile-time option."
+#endif
+                         );
+        }
     }
+
+    free(chatlist);
 }
 
 /* return length of password on success, 0 on failure */
@@ -369,12 +549,17 @@ static int password_prompt(char *buf, int size)
     }
 
     const char *p = fgets(buf, size, stdin);
-    int len = strlen(buf);
 
     /* re-enable terminal echo */
     tcsetattr(fileno(stdin), TCSANOW, &oflags);
 
-    if (p == NULL || len <= 1) {
+    if (p == NULL) {
+        return 0;
+    }
+
+    size_t len = strlen(buf);
+
+    if (len <= 1) {
         return 0;
     }
 
@@ -441,7 +626,7 @@ static void first_time_encrypt(const char *msg)
     char ch[256] = {0};
 
     do {
-        system("clear");
+        clear_screen();
         printf("%s ", msg);
         fflush(stdout);
 
@@ -500,7 +685,7 @@ static void first_time_encrypt(const char *msg)
         user_password.data_is_encrypted = true;
     }
 
-    system("clear");
+    clear_screen();
 }
 
 /* Store Tox profile data to path.
@@ -515,12 +700,19 @@ int store_data(Tox *m, const char *path)
         return -1;
     }
 
-    char temp_path[strlen(path) + strlen(TEMP_PROFILE_EXT) + 1];
-    snprintf(temp_path, sizeof(temp_path), "%s%s", path, TEMP_PROFILE_EXT);
+    size_t temp_buf_size = strlen(path) + strlen(TEMP_PROFILE_EXT) + 1;
+    char *temp_path = malloc(temp_buf_size);
+
+    if (temp_path == NULL) {
+        return -1;
+    }
+
+    snprintf(temp_path, temp_buf_size, "%s%s", path, TEMP_PROFILE_EXT);
 
     FILE *fp = fopen(temp_path, "wb");
 
     if (fp == NULL) {
+        free(temp_path);
         return -1;
     }
 
@@ -528,6 +720,7 @@ int store_data(Tox *m, const char *path)
     char *data = malloc(data_len * sizeof(char));
 
     if (data == NULL) {
+        free(temp_path);
         fclose(fp);
         return -1;
     }
@@ -540,17 +733,19 @@ int store_data(Tox *m, const char *path)
 
         if (enc_data == NULL) {
             fclose(fp);
+            free(temp_path);
             free(data);
             return -1;
         }
 
-        TOX_ERR_ENCRYPTION err;
+        Tox_Err_Encryption err;
         tox_pass_encrypt((uint8_t *) data, data_len, (uint8_t *) user_password.pass, user_password.len,
                          (uint8_t *) enc_data, &err);
 
         if (err != TOX_ERR_ENCRYPTION_OK) {
             fprintf(stderr, "tox_pass_encrypt() failed with error %d\n", err);
             fclose(fp);
+            free(temp_path);
             free(data);
             free(enc_data);
             return -1;
@@ -559,6 +754,7 @@ int store_data(Tox *m, const char *path)
         if (fwrite(enc_data, enc_len, 1, fp) != 1) {
             fprintf(stderr, "Failed to write profile data.\n");
             fclose(fp);
+            free(temp_path);
             free(data);
             free(enc_data);
             return -1;
@@ -569,6 +765,7 @@ int store_data(Tox *m, const char *path)
         if (fwrite(data, data_len, 1, fp) != 1) {
             fprintf(stderr, "Failed to write profile data.\n");
             fclose(fp);
+            free(temp_path);
             free(data);
             return -1;
         }
@@ -578,9 +775,12 @@ int store_data(Tox *m, const char *path)
     free(data);
 
     if (rename(temp_path, path) != 0) {
+        free(temp_path);
         return -1;
     }
 
+    free(temp_path);
+
     return 0;
 }
 
@@ -614,6 +814,15 @@ static void init_tox_options(struct Tox_Options *tox_opts)
     tox_options_set_udp_enabled(tox_opts, !arg_opts.force_tcp);
     tox_options_set_proxy_type(tox_opts, arg_opts.proxy_type);
     tox_options_set_tcp_port(tox_opts, arg_opts.tcp_port);
+    tox_options_set_local_discovery_enabled(tox_opts, !arg_opts.disable_local_discovery);
+
+    if (arg_opts.logging) {
+        tox_options_set_log_callback(tox_opts, cb_toxcore_logger);
+
+        if (arg_opts.log_fp != NULL) {
+            tox_options_set_log_user_data(tox_opts, arg_opts.log_fp);
+        }
+    }
 
     if (!tox_options_get_ipv6_enabled(tox_opts)) {
         queue_init_message("Forcing IPv4 connection");
@@ -660,10 +869,16 @@ static Tox *load_tox(char *data_path, struct Tox_Options *tox_opts, Tox_Err_New
             exit_toxic_err("failed in load_tox", FATALERR_FILEOP);
         }
 
-        char data[len];
+        char *data = malloc(len);
 
-        if (fread(data, sizeof(data), 1, fp) != 1) {
+        if (data == NULL) {
             fclose(fp);
+            exit_toxic_err("failed in load_tox", FATALERR_MEMORY);
+        }
+
+        if (fread(data, len, 1, fp) != 1) {
+            fclose(fp);
+            free(data);
             exit_toxic_err("failed in load_tox", FATALERR_FILEOP);
         }
 
@@ -672,6 +887,7 @@ static Tox *load_tox(char *data_path, struct Tox_Options *tox_opts, Tox_Err_New
         /* attempt to encrypt an already encrypted data file */
         if (arg_opts.encrypt_data && is_encrypted) {
             fclose(fp);
+            free(data);
             exit_toxic_err("failed in load_tox", FATALERR_ENCRYPT);
         }
 
@@ -690,12 +906,18 @@ static Tox *load_tox(char *data_path, struct Tox_Options *tox_opts, Tox_Err_New
             int pweval = user_settings->password_eval[0];
 
             if (!pweval) {
-                system("clear");   // TODO: is this portable?
+                clear_screen();
                 printf("Enter password (q to quit) ");
             }
 
             size_t plain_len = len - TOX_PASS_ENCRYPTION_EXTRA_LENGTH;
-            char plain[plain_len];
+            char *plain = malloc(plain_len);  // must be freed after tox_new()
+
+            if (plain == NULL) {
+                fclose(fp);
+                free(data);
+                exit_toxic_err("failed in load_tox", FATALERR_MEMORY);
+            }
 
             while (true) {
                 fflush(stdout); // Flush before prompts so the user sees the question/message
@@ -710,18 +932,20 @@ static Tox *load_tox(char *data_path, struct Tox_Options *tox_opts, Tox_Err_New
 
                 if (strcasecmp(user_password.pass, "q") == 0) {
                     fclose(fp);
+                    free(plain);
+                    free(data);
                     exit(0);
                 }
 
                 if (pwlen < MIN_PASSWORD_LEN) {
-                    system("clear");
+                    clear_screen();
                     sleep(1);
                     printf("Invalid password. Try again. ");
                     pweval = 0;
                     continue;
                 }
 
-                TOX_ERR_DECRYPTION pwerr;
+                Tox_Err_Decryption pwerr;
                 tox_pass_decrypt((uint8_t *) data, len, (uint8_t *) user_password.pass, pwlen,
                                  (uint8_t *) plain, &pwerr);
 
@@ -733,20 +957,26 @@ static Tox *load_tox(char *data_path, struct Tox_Options *tox_opts, Tox_Err_New
 
                     if (m == NULL) {
                         fclose(fp);
+                        free(data);
+                        free(plain);
                         return NULL;
                     }
 
                     break;
                 } else if (pwerr == TOX_ERR_DECRYPTION_FAILED) {
-                    system("clear");
+                    clear_screen();
                     sleep(1);
                     printf("Invalid password. Try again. ");
                     pweval = 0;
                 } else {
                     fclose(fp);
+                    free(data);
+                    free(plain);
                     exit_toxic_err("tox_pass_decrypt() failed", pwerr);
                 }
             }
+
+            free(plain);
         } else {   /* data is not encrypted */
             tox_options_set_savedata_type(tox_opts, TOX_SAVEDATA_TYPE_TOX_SAVE);
             tox_options_set_savedata_data(tox_opts, (uint8_t *) data, len);
@@ -755,11 +985,13 @@ static Tox *load_tox(char *data_path, struct Tox_Options *tox_opts, Tox_Err_New
 
             if (m == NULL) {
                 fclose(fp);
+                free(data);
                 return NULL;
             }
         }
 
         fclose(fp);
+        free(data);
     } else {   /* Data file does not/should not exist */
         if (file_exists(data_path)) {
             exit_toxic_err("failed in load_tox", FATALERR_FILEOP);
@@ -832,6 +1064,7 @@ static void do_toxic(Tox *m)
 
     tox_iterate(m, NULL);
     do_tox_connection(m);
+
     pthread_mutex_unlock(&Winthread.lock);
 }
 
@@ -870,20 +1103,18 @@ void *thread_cqueue(void *data)
     while (true) {
         pthread_mutex_lock(&Winthread.lock);
 
-        size_t i;
-
-        for (i = 2; i < MAX_WINDOWS_NUM; ++i) {
+        for (size_t i = 2; i < MAX_WINDOWS_NUM; ++i) {
             ToxWindow *toxwin = get_window_ptr(i);
 
-            if (toxwin != NULL && toxwin->is_chat
-                    && get_friend_connection_status(toxwin->num) != TOX_CONNECTION_NONE) {
+            if ((toxwin != NULL) && (toxwin->type == WINDOW_TYPE_CHAT)
+                    && (get_friend_connection_status(toxwin->num) != TOX_CONNECTION_NONE)) {
                 cqueue_try_send(toxwin, m);
             }
         }
 
         pthread_mutex_unlock(&Winthread.lock);
 
-        usleep(4000);
+        sleep_thread(750000L); // 0.75 seconds
     }
 }
 
@@ -897,7 +1128,8 @@ void *thread_av(void *data)
         toxav_iterate(av);
         pthread_mutex_unlock(&Winthread.lock);
 
-        usleep(toxav_iteration_interval(av) * 1000);
+        long int sleep_duration = toxav_iteration_interval(av) * 1000;
+        sleep_thread(sleep_duration);
     }
 }
 #endif /* AUDIO */
@@ -912,6 +1144,8 @@ static void print_usage(void)
     fprintf(stderr, "  -e, --encrypt-data       Encrypt an unencrypted data file\n");
     fprintf(stderr, "  -f, --file               Use specified data file\n");
     fprintf(stderr, "  -h, --help               Show this message and exit\n");
+    fprintf(stderr, "  -l, --logging            Enable toxcore logging: Requires [log_path | stderr]\n");
+    fprintf(stderr, "  -L, --no-lan             Disable local discovery\n");
     fprintf(stderr, "  -n, --nodes              Use specified DHTnodes file\n");
     fprintf(stderr, "  -o, --noconnect          Do not connect to the DHT network\n");
     fprintf(stderr, "  -p, --SOCKS5-proxy       Use SOCKS5 proxy: Requires [IP] [port]\n");
@@ -931,7 +1165,9 @@ static void print_version(void)
 
 static void set_default_opts(void)
 {
-    memset(&arg_opts, 0, sizeof(struct arg_opts));
+    arg_opts = (struct arg_opts) {
+        0
+    };
 
     /* set any non-zero defaults here*/
     arg_opts.proxy_type = TOX_PROXY_TYPE_NONE;
@@ -942,12 +1178,14 @@ static void parse_args(int argc, char *argv[])
     set_default_opts();
 
     static struct option long_opts[] = {
-        {"file", required_argument, 0, 'f'},
         {"ipv4", no_argument, 0, '4'},
         {"debug", no_argument, 0, 'b'},
         {"default-locale", no_argument, 0, 'd'},
         {"config", required_argument, 0, 'c'},
         {"encrypt-data", no_argument, 0, 'e'},
+        {"file", required_argument, 0, 'f'},
+        {"logging", required_argument, 0, 'l'},
+        {"no-lan", no_argument, 0, 'L'},
         {"nodes", required_argument, 0, 'n'},
         {"help", no_argument, 0, 'h'},
         {"noconnect", no_argument, 0, 'o'},
@@ -961,7 +1199,7 @@ static void parse_args(int argc, char *argv[])
         {NULL, no_argument, NULL, 0},
     };
 
-    const char *opts_str = "4bdehotuxvc:f:n:r:p:P:T:";
+    const char *opts_str = "4bdehLotuxvc:f:l:n:r:p:P:T:";
     int opt, indexptr;
     long int port = 0;
 
@@ -977,6 +1215,11 @@ static void parse_args(int argc, char *argv[])
                 break;
 
             case 'c':
+                if (optarg == NULL) {
+                    queue_init_message("Invalid argument for option: %d", opt);
+                    break;
+                }
+
                 snprintf(arg_opts.config_path, sizeof(arg_opts.config_path), "%s", optarg);
 
                 if (!file_exists(arg_opts.config_path)) {
@@ -995,23 +1238,31 @@ static void parse_args(int argc, char *argv[])
                 break;
 
             case 'f':
+                if (optarg == NULL) {
+                    queue_init_message("Invalid argument for option: %d", opt);
+                    break;
+                }
+
                 arg_opts.use_custom_data = 1;
 
                 if (DATA_FILE) {
                     free(DATA_FILE);
+                    DATA_FILE = NULL;
                 }
 
                 if (BLOCK_FILE) {
                     free(BLOCK_FILE);
+                    BLOCK_FILE = NULL;
                 }
 
                 DATA_FILE = malloc(strlen(optarg) + 1);
-                strcpy(DATA_FILE, optarg);
 
                 if (DATA_FILE == NULL) {
                     exit_toxic_err("failed in parse_args", FATALERR_MEMORY);
                 }
 
+                strcpy(DATA_FILE, optarg);
+
                 BLOCK_FILE = malloc(strlen(optarg) + strlen("-blocklist") + 1);
 
                 if (BLOCK_FILE == NULL) {
@@ -1025,7 +1276,38 @@ static void parse_args(int argc, char *argv[])
 
                 break;
 
+            case 'l':
+                if (optarg) {
+                    arg_opts.logging = true;
+
+                    if (strcmp(optarg, "stderr") != 0) {
+                        arg_opts.log_fp = fopen(optarg, "w");
+
+                        if (arg_opts.log_fp != NULL) {
+                            queue_init_message("Toxcore logging enabled to %s", optarg);
+                        } else {
+                            arg_opts.debug = true;
+                            queue_init_message("Failed to open log file %s. Falling back to stderr.", optarg);
+                        }
+                    } else {
+                        arg_opts.debug = true;
+                        queue_init_message("Toxcore logging enabled to stderr");
+                    }
+                }
+
+                break;
+
+            case 'L':
+                arg_opts.disable_local_discovery = 1;
+                queue_init_message("Local discovery disabled");
+                break;
+
             case 'n':
+                if (optarg == NULL) {
+                    queue_init_message("Invalid argument for option: %d", opt);
+                    break;
+                }
+
                 snprintf(arg_opts.nodes_path, sizeof(arg_opts.nodes_path), "%s", optarg);
                 break;
 
@@ -1035,6 +1317,11 @@ static void parse_args(int argc, char *argv[])
                 break;
 
             case 'p':
+                if (optarg == NULL) {
+                    queue_init_message("Invalid argument for option: %d", opt);
+                    break;
+                }
+
                 arg_opts.proxy_type = TOX_PROXY_TYPE_SOCKS5;
                 snprintf(arg_opts.proxy_address, sizeof(arg_opts.proxy_address), "%s", optarg);
 
@@ -1052,6 +1339,11 @@ static void parse_args(int argc, char *argv[])
                 break;
 
             case 'P':
+                if (optarg == NULL) {
+                    queue_init_message("Invalid argument for option: %d", opt);
+                    break;
+                }
+
                 arg_opts.proxy_type = TOX_PROXY_TYPE_HTTP;
                 snprintf(arg_opts.proxy_address, sizeof(arg_opts.proxy_address), "%s", optarg);
 
@@ -1069,6 +1361,11 @@ static void parse_args(int argc, char *argv[])
                 break;
 
             case 'r':
+                if (optarg == NULL) {
+                    queue_init_message("Invalid argument for option: %d", opt);
+                    break;
+                }
+
                 snprintf(arg_opts.nameserver_path, sizeof(arg_opts.nameserver_path), "%s", optarg);
 
                 if (!file_exists(arg_opts.nameserver_path)) {
@@ -1082,10 +1379,15 @@ static void parse_args(int argc, char *argv[])
                 break;
 
             case 'T':
+                if (optarg == NULL) {
+                    queue_init_message("Invalid argument for option: %d", opt);
+                    break;
+                }
+
                 port = strtol(optarg, NULL, 10);
 
                 if (port <= 0 || port > MAX_PORT_RANGE) {
-                    port = 14191;
+                    port = MAX_PORT_RANGE;
                 }
 
                 arg_opts.tcp_port = port;
@@ -1100,6 +1402,8 @@ static void parse_args(int argc, char *argv[])
                 exit(EXIT_SUCCESS);
 
             case 'h':
+
+            // Intentional fallthrough
             default:
                 print_usage();
                 exit(EXIT_SUCCESS);
@@ -1107,50 +1411,6 @@ static void parse_args(int argc, char *argv[])
     }
 }
 
-/* Looks for an old default profile data file and blocklist, and renames them to the new default names.
- *
- * Returns 0 on success.
- * Returns -1 on failure.
- */
-#define OLD_DATA_NAME "data"
-#define OLD_DATA_BLOCKLIST_NAME "data-blocklist"
-static int rename_old_profile(const char *user_config_dir)
-{
-    char old_data_file[strlen(user_config_dir) + strlen(CONFIGDIR) + strlen(OLD_DATA_NAME) + 1];
-    snprintf(old_data_file, sizeof(old_data_file), "%s%s%s", user_config_dir, CONFIGDIR, OLD_DATA_NAME);
-
-    if (!file_exists(old_data_file)) {
-        return 0;
-    }
-
-    if (file_exists(DATA_FILE)) {
-        return 0;
-    }
-
-    if (rename(old_data_file, DATA_FILE) != 0) {
-        return -1;
-    }
-
-    queue_init_message("Data file has been moved to %s", DATA_FILE);
-
-    char old_data_blocklist[strlen(user_config_dir) + strlen(CONFIGDIR) + strlen(OLD_DATA_BLOCKLIST_NAME) + 1];
-    snprintf(old_data_blocklist, sizeof(old_data_blocklist), "%s%s%s", user_config_dir, CONFIGDIR, OLD_DATA_BLOCKLIST_NAME);
-
-    if (!file_exists(old_data_blocklist)) {
-        return 0;
-    }
-
-    if (file_exists(BLOCK_FILE)) {
-        return 0;
-    }
-
-    if (rename(old_data_blocklist, BLOCK_FILE) != 0) {
-        return -1;
-    }
-
-    return 0;
-}
-
 /* Initializes the default config directory and data files used by toxic.
  *
  * Exits the process with an error on failure.
@@ -1193,34 +1453,21 @@ static void init_default_data_files(void)
         strcat(BLOCK_FILE, BLOCKNAME);
     }
 
-    /* For backwards compatibility with old toxic profile names. TODO: remove this some day */
-    if (rename_old_profile(user_config_dir) == -1) {
-        queue_init_message("Warning: Profile backwards compatibility failed.");
-    }
-
     free(user_config_dir);
 }
 
-// this doesn't do anything (yet)
-#ifdef X11
-void DnD_callback(const char *asdv, DropType dt)
-{
-    // if (dt != DT_plain)
-    //     return;
-
-    // pthread_mutex_lock(&Winthread.lock);
-    // line_info_add(prompt, NULL, NULL, NULL, SYS_MSG, 0, 0, asdv);
-    // pthread_mutex_unlock(&Winthread.lock);
-}
-#endif /* X11 */
-
 int main(int argc, char **argv)
 {
+    /* Make sure all written files are read/writeable only by the current user. */
+    umask(S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
+
     parse_args(argc, argv);
 
     /* Use the -b flag to enable stderr */
     if (!arg_opts.debug) {
-        freopen("/dev/null", "w", stderr);
+        if (!freopen("/dev/null", "w", stderr)) {
+            fprintf(stderr, "Warning: failed to enable stderr\n");
+        }
     }
 
     if (arg_opts.encrypt_data && arg_opts.unencrypt_data) {
@@ -1229,9 +1476,6 @@ int main(int argc, char **argv)
         queue_init_message("Warning: Using --unencrypt-data and --encrypt-data simultaneously has no effect");
     }
 
-    /* Make sure all written files are read/writeable only by the current user. */
-    umask(S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
-
     init_default_data_files();
 
     bool datafile_exists = file_exists(DATA_FILE);
@@ -1269,7 +1513,7 @@ int main(int argc, char **argv)
 
 #ifdef X11
 
-    if (init_xtra(DnD_callback) == -1) {
+    if (init_x11focus() == -1) {
         queue_init_message("X failed to initialize");
     }
 
@@ -1285,22 +1529,13 @@ int main(int argc, char **argv)
 
     prompt = init_windows(m);
     prompt_init_statusbar(prompt, m, !datafile_exists);
-    load_groups(prompt, m);
+    load_conferences(m);
+    set_active_window_index(0);
 
-    /* thread for ncurses stuff */
     if (pthread_mutex_init(&Winthread.lock, NULL) != 0) {
         exit_toxic_err("failed in main", FATALERR_MUTEX_INIT);
     }
 
-    if (pthread_create(&Winthread.tid, NULL, thread_winref, (void *) m) != 0) {
-        exit_toxic_err("failed in main", FATALERR_THREAD_CREATE);
-    }
-
-    /* thread for message queue */
-    if (pthread_create(&cqueue_thread.tid, NULL, thread_cqueue, (void *) m) != 0) {
-        exit_toxic_err("failed in main", FATALERR_THREAD_CREATE);
-    }
-
 #ifdef AUDIO
 
     av = init_audio(prompt, m);
@@ -1315,8 +1550,8 @@ int main(int argc, char **argv)
         exit_toxic_err("failed in main", FATALERR_THREAD_CREATE);
     }
 
-    set_primary_device(input, user_settings->audio_in_dev);
-    set_primary_device(output, user_settings->audio_out_dev);
+    set_al_device(input, user_settings->audio_in_dev);
+    set_al_device(output, user_settings->audio_out_dev);
 
 #elif SOUND_NOTIFY
 
@@ -1326,6 +1561,16 @@ int main(int argc, char **argv)
 
 #endif /* AUDIO */
 
+    /* thread for ncurses stuff */
+    if (pthread_create(&Winthread.tid, NULL, thread_winref, (void *) m) != 0) {
+        exit_toxic_err("failed in main", FATALERR_THREAD_CREATE);
+    }
+
+    /* thread for message queue */
+    if (pthread_create(&cqueue_thread.tid, NULL, thread_cqueue, (void *) m) != 0) {
+        exit_toxic_err("failed in main", FATALERR_THREAD_CREATE);
+    }
+
 #ifdef PYTHON
 
     init_python(m);
@@ -1333,7 +1578,7 @@ int main(int argc, char **argv)
 
 #endif /* PYTHON */
 
-    init_notify(60, 3000);
+    init_notify(60, user_settings->notification_timeout);
 
     /* screen/tmux auto-away timer */
     if (init_mplex_away_timer(m) == -1) {
@@ -1364,11 +1609,11 @@ int main(int argc, char **argv)
 
         time_t cur_time = get_unix_time();
 
-        if (timed_out(last_save, AUTOSAVE_FREQ)) {
+        if (user_settings->autosave_freq > 0 && timed_out(last_save, user_settings->autosave_freq)) {
             pthread_mutex_lock(&Winthread.lock);
 
             if (store_data(m, DATA_FILE) != 0) {
-                line_info_add(prompt, NULL, NULL, NULL, SYS_MSG, 0, RED, "WARNING: Failed to save to data file");
+                line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, RED, "WARNING: Failed to save to data file");
             }
 
             pthread_mutex_unlock(&Winthread.lock);
@@ -1376,7 +1621,8 @@ int main(int argc, char **argv)
             last_save = cur_time;
         }
 
-        usleep(tox_iteration_interval(m) * 1000);
+        long int sleep_duration = tox_iteration_interval(m) * 1000;
+        sleep_thread(sleep_duration);
     }
 
     return 0;
