@@ -48,6 +48,7 @@
 #define TOXIC_MAX_NAME_LENGTH 32   /* Must be <= TOX_MAX_NAME_LENGTH */
 #define KEY_IDENT_DIGITS 3    /* number of hex digits to display for the pub-key based identifier */
 #define TIME_STR_SIZE 32
+#define COLOR_STR_SIZE 10 /* should fit every color option */
 
 #ifndef MAX_PORT_RANGE
 #define MAX_PORT_RANGE 65535
@@ -61,8 +62,6 @@
 #define T_KEY_PREV       0x0F     /* ctrl-o */
 #define T_KEY_C_E        0x05     /* ctrl-e */
 #define T_KEY_C_A        0x01     /* ctrl-a */
-#define T_KEY_C_RB       0x1D     /* ctrl-] */
-#define T_KEY_C_LB       0x1B     /* ctrl-[ */
 #define T_KEY_C_V        0x16     /* ctrl-v */
 #define T_KEY_C_F        0x06     /* ctrl-f */
 #define T_KEY_C_H        0x08     /* ctrl-h */
@@ -71,9 +70,13 @@
 #define T_KEY_C_W        0x17     /* ctrl-w */
 #define T_KEY_C_B        0x02     /* ctrl-b */
 #define T_KEY_C_T        0x14     /* ctrl-t */
+#define T_KEY_C_LEFT     0x221    /* ctrl-left arrow */
+#define T_KEY_C_RIGHT    0x230    /* ctrl-right arrow */
+#define T_KEY_C_UP       0x236    /* ctrl-up arrow */
+#define T_KEY_C_DOWN     0x20D    /* ctrl-down arrow */
 #define T_KEY_TAB        0x09     /* TAB key */
 
-#define ONLINE_CHAR "*"
+#define ONLINE_CHAR  "o"
 #define OFFLINE_CHAR "o"
 
 typedef enum _FATAL_ERRS {
@@ -113,14 +116,14 @@ void on_friend_name(Tox *m, uint32_t friendnumber, const uint8_t *string, size_t
 void on_friend_status(Tox *m, uint32_t friendnumber, Tox_User_Status status, void *userdata);
 void on_friend_status_message(Tox *m, uint32_t friendnumber, const uint8_t *string, size_t length, void *userdata);
 void on_friend_added(Tox *m, uint32_t friendnumber, bool sort);
-void on_conference_message(Tox *m, uint32_t groupnumber, uint32_t peernumber, Tox_Message_Type type,
+void on_conference_message(Tox *m, uint32_t conferencenumber, uint32_t peernumber, Tox_Message_Type type,
                            const uint8_t *message, size_t length, void *userdata);
-void on_conference_invite(Tox *m, uint32_t friendnumber, Tox_Conference_Type type, const uint8_t *group_pub_key,
+void on_conference_invite(Tox *m, uint32_t friendnumber, Tox_Conference_Type type, const uint8_t *conference_pub_key,
                           size_t length, void *userdata);
-void on_conference_peer_list_changed(Tox *m, uint32_t groupnumber, void *userdata);
-void on_conference_peer_name(Tox *m, uint32_t groupnumber, uint32_t peernumber, const uint8_t *name,
+void on_conference_peer_list_changed(Tox *m, uint32_t conferencenumber, void *userdata);
+void on_conference_peer_name(Tox *m, uint32_t conferencenumber, uint32_t peernumber, const uint8_t *name,
                              size_t length, void *userdata);
-void on_conference_title(Tox *m, uint32_t groupnumber, uint32_t peernumber, const uint8_t *title, size_t length,
+void on_conference_title(Tox *m, uint32_t conferencenumber, uint32_t peernumber, const uint8_t *title, size_t length,
                          void *userdata);
 void on_file_chunk_request(Tox *m, uint32_t friendnumber, uint32_t filenumber, uint64_t position, size_t length,
                            void *userdata);
