@@ -20,22 +20,22 @@
  *
  */
 
+#include <ctype.h>
+#include <pthread.h>
 #include <stdlib.h>
 #include <string.h>
-#include <pthread.h>
-#include <ctype.h>
 
-#include "friendlist.h"
-#include "prompt.h"
-#include "toxic.h"
-#include "windows.h"
-#include "groupchat.h"
+#include "avatars.h"
 #include "chat.h"
+#include "conference.h"
+#include "file_transfers.h"
+#include "friendlist.h"
 #include "line_info.h"
 #include "misc_tools.h"
-#include "avatars.h"
+#include "prompt.h"
 #include "settings.h"
-#include "file_transfers.h"
+#include "toxic.h"
+#include "windows.h"
 
 extern char *DATA_FILE;
 extern struct Winthread Winthread;
@@ -50,6 +50,8 @@ extern struct user_settings *user_settings;
 /* CALLBACKS START */
 void on_friend_request(Tox *m, const uint8_t *public_key, const uint8_t *data, size_t length, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     char msg[MAX_STR_SIZE + 1];
     length = copy_tox_str(msg, sizeof(msg), (const char *) data, length);
 
@@ -62,6 +64,10 @@ void on_friend_request(Tox *m, const uint8_t *public_key, const uint8_t *data, s
 
 void on_friend_connection_status(Tox *m, uint32_t friendnumber, Tox_Connection connection_status, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
+    on_avatar_friend_connection_status(m, friendnumber, connection_status);
+
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onConnectionChange != NULL) {
             windows[i]->onConnectionChange(windows[i], m, friendnumber, connection_status);
@@ -71,6 +77,8 @@ void on_friend_connection_status(Tox *m, uint32_t friendnumber, Tox_Connection c
 
 void on_friend_typing(Tox *m, uint32_t friendnumber, bool is_typing, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     if (user_settings->show_typing_other == SHOW_TYPING_OFF) {
         return;
     }
@@ -85,6 +93,8 @@ void on_friend_typing(Tox *m, uint32_t friendnumber, bool is_typing, void *userd
 void on_friend_message(Tox *m, uint32_t friendnumber, Tox_Message_Type type, const uint8_t *string, size_t length,
                        void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     char msg[MAX_STR_SIZE + 1];
     length = copy_tox_str(msg, sizeof(msg), (const char *) string, length);
 
@@ -97,6 +107,8 @@ void on_friend_message(Tox *m, uint32_t friendnumber, Tox_Message_Type type, con
 
 void on_friend_name(Tox *m, uint32_t friendnumber, const uint8_t *string, size_t length, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     char nick[TOXIC_MAX_NAME_LENGTH + 1];
     length = copy_tox_str(nick, sizeof(nick), (const char *) string, length);
     filter_str(nick, length);
@@ -112,6 +124,9 @@ void on_friend_name(Tox *m, uint32_t friendnumber, const uint8_t *string, size_t
 
 void on_friend_status_message(Tox *m, uint32_t friendnumber, const uint8_t *string, size_t length, void *userdata)
 {
+    UNUSED_VAR(userdata);
+    UNUSED_VAR(m);
+
     char msg[TOX_MAX_STATUS_MESSAGE_LENGTH + 1];
     length = copy_tox_str(msg, sizeof(msg), (const char *) string, length);
     filter_str(msg, length);
@@ -125,6 +140,8 @@ void on_friend_status_message(Tox *m, uint32_t friendnumber, const uint8_t *stri
 
 void on_friend_status(Tox *m, uint32_t friendnumber, Tox_User_Status status, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onStatusChange != NULL) {
             windows[i]->onStatusChange(windows[i], m, friendnumber, status);
@@ -143,61 +160,71 @@ void on_friend_added(Tox *m, uint32_t friendnumber, bool sort)
     store_data(m, DATA_FILE);
 }
 
-void on_conference_message(Tox *m, uint32_t groupnumber, uint32_t peernumber, Tox_Message_Type type,
+void on_conference_message(Tox *m, uint32_t conferencenumber, uint32_t peernumber, Tox_Message_Type type,
                            const uint8_t *message, size_t length, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     char msg[MAX_STR_SIZE + 1];
     length = copy_tox_str(msg, sizeof(msg), (const char *) message, length);
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onGroupMessage != NULL) {
-            windows[i]->onGroupMessage(windows[i], m, groupnumber, peernumber, type, msg, length);
+        if (windows[i] != NULL && windows[i]->onConferenceMessage != NULL) {
+            windows[i]->onConferenceMessage(windows[i], m, conferencenumber, peernumber, type, msg, length);
         }
     }
 }
 
-void on_conference_invite(Tox *m, uint32_t friendnumber, Tox_Conference_Type type, const uint8_t *group_pub_key,
+void on_conference_invite(Tox *m, uint32_t friendnumber, Tox_Conference_Type type, const uint8_t *conference_pub_key,
                           size_t length, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onGroupInvite != NULL) {
-            windows[i]->onGroupInvite(windows[i], m, friendnumber, type, (char *) group_pub_key, length);
+        if (windows[i] != NULL && windows[i]->onConferenceInvite != NULL) {
+            windows[i]->onConferenceInvite(windows[i], m, friendnumber, type, (char *) conference_pub_key, length);
         }
     }
 }
 
-void on_conference_peer_list_changed(Tox *m, uint32_t groupnumber, void *userdata)
+void on_conference_peer_list_changed(Tox *m, uint32_t conferencenumber, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onGroupNameListChange != NULL) {
-            windows[i]->onGroupNameListChange(windows[i], m, groupnumber);
+        if (windows[i] != NULL && windows[i]->onConferenceNameListChange != NULL) {
+            windows[i]->onConferenceNameListChange(windows[i], m, conferencenumber);
         }
     }
 }
 
-void on_conference_peer_name(Tox *m, uint32_t groupnumber, uint32_t peernumber, const uint8_t *name,
+void on_conference_peer_name(Tox *m, uint32_t conferencenumber, uint32_t peernumber, const uint8_t *name,
                              size_t length, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     char nick[TOXIC_MAX_NAME_LENGTH + 1];
     length = copy_tox_str(nick, sizeof(nick), (const char *) name, length);
     filter_str(nick, length);
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onGroupPeerNameChange != NULL) {
-            windows[i]->onGroupPeerNameChange(windows[i], m, groupnumber, peernumber, nick, length);
+        if (windows[i] != NULL && windows[i]->onConferencePeerNameChange != NULL) {
+            windows[i]->onConferencePeerNameChange(windows[i], m, conferencenumber, peernumber, nick, length);
         }
     }
 }
 
-void on_conference_title(Tox *m, uint32_t groupnumber, uint32_t peernumber, const uint8_t *title, size_t length,
+void on_conference_title(Tox *m, uint32_t conferencenumber, uint32_t peernumber, const uint8_t *title, size_t length,
                          void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     char data[MAX_STR_SIZE + 1];
     length = copy_tox_str(data, sizeof(data), (const char *) title, length);
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onGroupTitleChange != NULL) {
-            windows[i]->onGroupTitleChange(windows[i], m, groupnumber, peernumber, data, length);
+        if (windows[i] != NULL && windows[i]->onConferenceTitleChange != NULL) {
+            windows[i]->onConferenceTitleChange(windows[i], m, conferencenumber, peernumber, data, length);
         }
     }
 }
@@ -205,6 +232,8 @@ void on_conference_title(Tox *m, uint32_t groupnumber, uint32_t peernumber, cons
 void on_file_chunk_request(Tox *m, uint32_t friendnumber, uint32_t filenumber, uint64_t position,
                            size_t length, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     struct FileTransfer *ft = get_file_transfer_struct(friendnumber, filenumber);
 
     if (!ft) {
@@ -224,8 +253,10 @@ void on_file_chunk_request(Tox *m, uint32_t friendnumber, uint32_t filenumber, u
 }
 
 void on_file_recv_chunk(Tox *m, uint32_t friendnumber, uint32_t filenumber, uint64_t position,
-                        const uint8_t *data, size_t length, void *user_data)
+                        const uint8_t *data, size_t length, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     struct FileTransfer *ft = get_file_transfer_struct(friendnumber, filenumber);
 
     if (!ft) {
@@ -242,6 +273,8 @@ void on_file_recv_chunk(Tox *m, uint32_t friendnumber, uint32_t filenumber, uint
 void on_file_recv_control(Tox *m, uint32_t friendnumber, uint32_t filenumber, Tox_File_Control control,
                           void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     struct FileTransfer *ft = get_file_transfer_struct(friendnumber, filenumber);
 
     if (!ft) {
@@ -263,6 +296,8 @@ void on_file_recv_control(Tox *m, uint32_t friendnumber, uint32_t filenumber, To
 void on_file_recv(Tox *m, uint32_t friendnumber, uint32_t filenumber, uint32_t kind, uint64_t file_size,
                   const uint8_t *filename, size_t filename_length, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     /* We don't care about receiving avatars */
     if (kind != TOX_FILE_KIND_DATA) {
         tox_file_control(m, friendnumber, filenumber, TOX_FILE_CONTROL_CANCEL, NULL);
@@ -279,6 +314,8 @@ void on_file_recv(Tox *m, uint32_t friendnumber, uint32_t filenumber, uint32_t k
 
 void on_friend_read_receipt(Tox *m, uint32_t friendnumber, uint32_t receipt, void *userdata)
 {
+    UNUSED_VAR(userdata);
+
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onReadReceipt != NULL) {
             windows[i]->onReadReceipt(windows[i], m, friendnumber, receipt);
@@ -299,7 +336,7 @@ int add_window(Tox *m, ToxWindow *w)
         }
 
         w->index = i;
-        w->window = newwin(LINES - 2, COLS, 0, 0);
+        w->window = newwin(LINES, COLS, 0, 0);
 
         if (w->window == NULL) {
             return -1;
@@ -360,8 +397,9 @@ void del_window(ToxWindow *w)
     set_active_window_index(0);
 
     uint8_t idx = w->index;
+    delwin(w->window_bar);
     delwin(w->window);
-    free(windows[idx]);
+    free(w);
     windows[idx] = NULL;
 
     clear();
@@ -371,7 +409,12 @@ void del_window(ToxWindow *w)
 
 ToxWindow *init_windows(Tox *m)
 {
+    if (COLS <= CHATBOX_HEIGHT + WINDOW_BAR_HEIGHT) {
+        exit_toxic_err("add_window() for prompt failed in init_windows", FATALERR_WININIT);
+    }
+
     prompt = new_prompt();
+
     int n_prompt = add_window(m, prompt);
 
     if (n_prompt < 0) {
@@ -393,25 +436,18 @@ void on_window_resize(void)
     refresh();
     clear();
 
-    /* equivalent to LINES and COLS */
-    int x2, y2;
-    getmaxyx(stdscr, y2, x2);
-    y2 -= 2;
-
-    if (y2 <= 0 || x2 <= 0) {
-        return;
-    }
-
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] == NULL) {
+        ToxWindow *w = windows[i];
+
+        if (w == NULL) {
             continue;
         }
 
-        ToxWindow *w = windows[i];
-
-        if (windows[i]->is_friendlist)  {
+        if (w->type == WINDOW_TYPE_FRIEND_LIST)  {
+            delwin(w->window_bar);
             delwin(w->window);
-            w->window = newwin(y2, x2, 0, 0);
+            w->window = newwin(LINES, COLS, 0, 0);
+            w->window_bar = subwin(w->window, WINDOW_BAR_HEIGHT, COLS, LINES - 2, 0);
             continue;
         }
 
@@ -419,7 +455,7 @@ void on_window_resize(void)
             wclear(w->help->win);
         }
 
-        if (w->is_groupchat) {
+        if (w->type == WINDOW_TYPE_CONFERENCE) {
             delwin(w->chatwin->sidebar);
             w->chatwin->sidebar = NULL;
         } else {
@@ -428,22 +464,35 @@ void on_window_resize(void)
 
         delwin(w->chatwin->linewin);
         delwin(w->chatwin->history);
+        delwin(w->window_bar);
         delwin(w->window);
 
-        w->window = newwin(y2, x2, 0, 0);
-        w->chatwin->linewin = subwin(w->window, CHATBOX_HEIGHT, x2, y2 - CHATBOX_HEIGHT, 0);
+        w->window = newwin(LINES, COLS, 0, 0);
+
+        int x2;
+        int y2;
+        getmaxyx(w->window, y2, x2);
+
+        if (y2 <= 0 || x2 <= 0) {
+            fprintf(stderr, "Failed to resize window: max_x: %d, max_y: %d\n", x2, y2);
+            delwin(w->window);
+            return;
+        }
 
         if (w->show_peerlist) {
-            w->chatwin->history = subwin(w->window, y2 - CHATBOX_HEIGHT + 1, x2 - SIDEBAR_WIDTH - 1, 0, 0);
-            w->chatwin->sidebar = subwin(w->window, y2 - CHATBOX_HEIGHT + 1, SIDEBAR_WIDTH, 0, x2 - SIDEBAR_WIDTH);
+            w->chatwin->history = subwin(w->window, y2 - CHATBOX_HEIGHT - WINDOW_BAR_HEIGHT, x2 - SIDEBAR_WIDTH - 1, 0, 0);
+            w->chatwin->sidebar = subwin(w->window, y2 - CHATBOX_HEIGHT - WINDOW_BAR_HEIGHT, SIDEBAR_WIDTH, 0, x2 - SIDEBAR_WIDTH);
         } else {
-            w->chatwin->history = subwin(w->window, y2 - CHATBOX_HEIGHT + 1, x2, 0, 0);
+            w->chatwin->history =  subwin(w->window, y2 - CHATBOX_HEIGHT - WINDOW_BAR_HEIGHT, x2, 0, 0);
 
-            if (!w->is_groupchat) {
-                w->stb->topline = subwin(w->window, 2, x2, 0, 0);
+            if (w->type != WINDOW_TYPE_CONFERENCE) {
+                w->stb->topline = subwin(w->window, TOP_BAR_HEIGHT, x2, 0, 0);
             }
         }
 
+        w->window_bar = subwin(w->window, WINDOW_BAR_HEIGHT, x2, y2 - (CHATBOX_HEIGHT + WINDOW_BAR_HEIGHT), 0);
+        w->chatwin->linewin = subwin(w->window, CHATBOX_HEIGHT, x2, y2 - WINDOW_BAR_HEIGHT, 0);
+
 #ifdef AUDIO
 
         if (w->chatwin->infobox.active) {
@@ -454,82 +503,190 @@ void on_window_resize(void)
 #endif /* AUDIO */
 
         scrollok(w->chatwin->history, 0);
+        wmove(w->window, y2 - CURS_Y_OFFSET, 0);
     }
 }
 
-static void draw_window_tab(ToxWindow *toxwin)
+static void draw_window_tab(WINDOW *win, ToxWindow *toxwin, bool active_window)
 {
     pthread_mutex_lock(&Winthread.lock);
 
-    if (toxwin->alert != WINDOW_ALERT_NONE) {
-        attron(COLOR_PAIR(toxwin->alert));
-    }
+    bool has_alert = toxwin->alert != WINDOW_ALERT_NONE;
+    unsigned int pending_messages = toxwin->pending_messages;
 
     pthread_mutex_unlock(&Winthread.lock);
 
-    clrtoeol();
-    printw(" [%s]", toxwin->name);
+    WINDOW_TYPE type = toxwin->type;
 
-    pthread_mutex_lock(&Winthread.lock);
-
-    if (toxwin->alert != WINDOW_ALERT_NONE) {
-        attroff(COLOR_PAIR(toxwin->alert));
+    if (active_window) {
+        wattron(win, A_BOLD | COLOR_PAIR(BAR_ACCENT));
+        wprintw(win, " [");
+        wattroff(win, COLOR_PAIR(BAR_ACCENT));
+        wattron(win, COLOR_PAIR(BAR_TEXT));
+    } else {
+        if (has_alert) {
+            wattron(win, COLOR_PAIR(BAR_ACCENT));
+            wprintw(win, " [");
+            wattroff(win, COLOR_PAIR(BAR_ACCENT));
+            wattron(win, A_BOLD | COLOR_PAIR(toxwin->alert));
+        } else {
+            wattron(win, COLOR_PAIR(BAR_ACCENT));
+            wprintw(win, " [");
+            wattroff(win, COLOR_PAIR(BAR_ACCENT));
+            wattron(win, COLOR_PAIR(BAR_TEXT));
+        }
     }
 
-    pthread_mutex_unlock(&Winthread.lock);
+    if (active_window || (type == WINDOW_TYPE_PROMPT || type == WINDOW_TYPE_FRIEND_LIST)) {
+        wprintw(win, "%s", toxwin->name);
+    } else {
+        if (pending_messages > 0) {
+            wprintw(win, "%u", pending_messages);
+        } else {
+            wprintw(win, "-");
+        }
+    }
+
+    if (active_window) {
+        wattroff(win, COLOR_PAIR(BAR_TEXT));
+        wattron(win, COLOR_PAIR(BAR_ACCENT));
+        wprintw(win, "]");
+        wattroff(win, A_BOLD | COLOR_PAIR(BAR_ACCENT));
+    } else {
+        if (has_alert) {
+            wattroff(win, A_BOLD | COLOR_PAIR(toxwin->alert));
+            wattron(win, COLOR_PAIR(BAR_ACCENT));
+            wprintw(win, "]");
+            wattroff(win, COLOR_PAIR(BAR_ACCENT));
+        } else {
+            wattroff(win, COLOR_PAIR(BAR_TEXT));
+            wattron(win, COLOR_PAIR(BAR_ACCENT));
+            wprintw(win, "]");
+            wattroff(win, COLOR_PAIR(BAR_ACCENT));
+        }
+    }
 }
 
-static void draw_bar(void)
+void draw_window_bar(ToxWindow *self)
 {
-    int y, x;
+    WINDOW *win = self->window_bar;
+    wclear(win);
 
-    ToxWindow *w = windows[active_window_index];
-
-    if (w == NULL) {
-        return;
+    if (self->scroll_pause) {
+        wattron(win, A_BLINK | A_BOLD | COLOR_PAIR(BAR_NOTIFY));
+        wprintw(win, "^");
+        wattroff(win, A_BLINK | A_BOLD | COLOR_PAIR(BAR_NOTIFY));
+    } else {
+        wattron(win, COLOR_PAIR(BAR_TEXT));
+        wprintw(win, " ");
+        wattroff(win, COLOR_PAIR(BAR_TEXT));
     }
 
-    // save current cursor position
-    getyx(w->window, y, x);
-
-    attron(COLOR_PAIR(BLUE));
-    mvhline(LINES - 2, 0, '_', COLS);
-    attroff(COLOR_PAIR(BLUE));
-
-    move(LINES - 1, 0);
-
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] == NULL) {
             continue;
         }
 
-        if (i == active_window_index) {
+        bool active_window = i == active_window_index;
+        draw_window_tab(win, windows[i], active_window);
+    }
 
-#ifdef URXVT_FIX
-            attron(A_BOLD | COLOR_PAIR(GREEN));
-        } else {
-#endif
+    int cur_x;
+    int cur_y;
+    UNUSED_VAR(cur_y);
 
-            attron(A_BOLD);
+    getyx(win, cur_y, cur_x);
+
+    wattron(win, COLOR_PAIR(BAR_TEXT));
+    mvwhline(win, 0, cur_x, ' ', COLS - cur_x);
+    wattroff(win, COLOR_PAIR(BAR_TEXT));
+}
+
+/*
+ * Gets current char from stdscr and puts it in ch.
+ *
+ * Return 1 if char is printable.
+ * Return 0 if char is not printable.
+ * Return -1 on error.
+ */
+static int get_current_char(wint_t *ch)
+{
+    wint_t tmpchar = 0;
+    bool is_printable = false;
+
+#ifdef HAVE_WIDECHAR
+    int status = wget_wch(stdscr, &tmpchar);
+
+    if (status == ERR) {
+        return -1;
+    }
+
+    if (status == OK) {
+        is_printable = iswprint(tmpchar);
+    }
+
+#else
+    tmpchar = getch();
+
+    if (tmpchar == ERR) {
+        return -1;
+    }
+
+    is_printable = isprint(tmpchar);
+#endif /* HAVE_WIDECHAR */
+
+    *ch = tmpchar;
+
+    return (int) is_printable;
+}
+
+static struct key_sequence_codes {
+    wchar_t *code;
+    wint_t   key;
+} Keys[] = {
+    { L"[1;5A", T_KEY_C_UP    },
+    { L"[1;5B", T_KEY_C_DOWN  },
+    { L"[1;5C", T_KEY_C_RIGHT },
+    { L"[1;5D", T_KEY_C_LEFT  },
+    { NULL, 0 }
+};
+
+/*
+ * Return key code corresponding to character sequence queued in stdscr.
+ * Return -1 if sequence is unknown.
+ */
+#define MAX_SEQUENCE_SIZE 5
+static wint_t get_input_sequence_code(void)
+{
+    wchar_t code[MAX_SEQUENCE_SIZE + 1];
+
+    size_t length = 0;
+    wint_t ch = 0;
+
+    for (size_t i = 0; i < MAX_SEQUENCE_SIZE; ++i) {
+        int res = get_current_char(&ch);
+
+        if (res < 0) {
+            break;
         }
 
-        draw_window_tab(windows[i]);
+        ++length;
+        code[i] = (wchar_t) ch;
+    }
 
-        if (i == active_window_index) {
+    if (length == 0) {
+        return -1;
+    }
 
-#ifdef URXVT_FIX
-            attroff(A_BOLD | COLOR_PAIR(GREEN));
-        } else {
-#endif
+    code[length] = L'\0';
 
-            attroff(A_BOLD);
+    for (size_t i = 0; Keys[i].key != 0; ++i) {
+        if (wcscmp(code, Keys[i].code) == 0) {
+            return Keys[i].key;
         }
     }
 
-    // restore cursor position after drawing
-    move(y, x);
-
-    refresh();
+    return -1;
 }
 
 void draw_active_window(Tox *m)
@@ -542,53 +699,48 @@ void draw_active_window(Tox *m)
 
     pthread_mutex_lock(&Winthread.lock);
     a->alert = WINDOW_ALERT_NONE;
+    a->pending_messages = 0;
     pthread_mutex_unlock(&Winthread.lock);
 
-    wint_t ch = 0;
-
-    draw_bar();
-
     touchwin(a->window);
     a->onDraw(a, m);
     wrefresh(a->window);
 
-    /* Handle input */
-    bool ltr;
-#ifdef HAVE_WIDECHAR
-    int status = wget_wch(stdscr, &ch);
+    wint_t ch = 0;
+    int printable = get_current_char(&ch);
 
-    if (status == ERR) {
+    if (printable < 0) {
         return;
     }
 
-    if (status == OK) {
-        ltr = iswprint(ch);
-    } else { /* if (status == KEY_CODE_YES) */
-        ltr = false;
-    }
-
-#else
-    ch = getch();
-
-    if (ch == ERR) {
-        return;
-    }
-
-    /* TODO verify if this works */
-    ltr = isprint(ch);
-#endif /* HAVE_WIDECHAR */
-
-    if (!ltr && (ch == user_settings->key_next_tab || ch == user_settings->key_prev_tab)) {
+    if (printable == 0 && (ch == user_settings->key_next_tab || ch == user_settings->key_prev_tab)) {
         set_next_window((int) ch);
-    } else {
+        return;
+    } else if ((printable == 0) && (a->type != WINDOW_TYPE_FRIEND_LIST)) {
         pthread_mutex_lock(&Winthread.lock);
-        a->onKey(a, m, ch, ltr);
+        bool input_ret = a->onKey(a, m, ch, (bool) printable);
         pthread_mutex_unlock(&Winthread.lock);
+
+        if (input_ret) {
+            return;
+        }
+
+        // if an unprintable key code is unrecognized by input handler we attempt to manually decode char sequence
+        wint_t tmp = get_input_sequence_code();
+
+        if (tmp != (wint_t) -1) {
+            ch = tmp;
+        }
     }
+
+    pthread_mutex_lock(&Winthread.lock);
+    a->onKey(a, m, ch, (bool) printable);
+    pthread_mutex_unlock(&Winthread.lock);
 }
 
-/* refresh inactive windows to prevent scrolling bugs.
-   call at least once per second */
+/* Refresh inactive windows to prevent scrolling bugs.
+ * Call at least once per second.
+ */
 void refresh_inactive_windows(void)
 {
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
@@ -598,7 +750,7 @@ void refresh_inactive_windows(void)
             continue;
         }
 
-        if (i != active_window_index && !toxwin->is_friendlist) {
+        if ((i != active_window_index) && (toxwin->type != WINDOW_TYPE_FRIEND_LIST)) {
             pthread_mutex_lock(&Winthread.lock);
             line_info_print(toxwin);
             pthread_mutex_unlock(&Winthread.lock);
@@ -636,7 +788,7 @@ int get_num_active_windows(void)
     return num_active_windows;
 }
 
-/* destroys all chat and groupchat windows (should only be called on shutdown) */
+/* destroys all chat and conference windows (should only be called on shutdown) */
 void kill_all_windows(Tox *m)
 {
     for (uint8_t i = 2; i < MAX_WINDOWS_NUM; ++i) {
@@ -644,10 +796,10 @@ void kill_all_windows(Tox *m)
             continue;
         }
 
-        if (windows[i]->is_chat) {
+        if (windows[i]->type == WINDOW_TYPE_CHAT) {
             kill_chat_window(windows[i], m);
-        } else if (windows[i]->is_groupchat) {
-            free_groupchat(windows[i], m, windows[i]->num);
+        } else if (windows[i]->type == WINDOW_TYPE_CONFERENCE) {
+            free_conference(windows[i], windows[i]->num);
         }
     }
 
